@@ -22,23 +22,17 @@
  */
 #include "config.h"
 
-#include <errno.h>
-#include <fcntl.h>
-#include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <stddef.h>
 #include <string.h>
 #include <unistd.h>
+#include <fcntl.h>
+#include <errno.h>
 #include <sys/socket.h>
-#include <sys/syscall.h>
 #include <sys/un.h>
 
 #include "log.h"
-#include "utils.h"
-
-#ifndef HAVE_STRLCPY
-#include "include/strlcpy.h"
-#endif
 
 lxc_log_define(lxc_af_unix, lxc);
 
@@ -67,9 +61,8 @@ int lxc_abstract_unix_open(const char *path, int type, int flags)
 		errno = ENAMETOOLONG;
 		return -1;
 	}
-
-	/* do not enforce \0-termination */
-	memcpy(&addr.sun_path[1], &path[1], len);
+	/* addr.sun_path[0] has already been set to 0 by memset() */
+	strncpy(&addr.sun_path[1], &path[1], strlen(&path[1]));
 
 	ret = bind(fd, (struct sockaddr *)&addr,
 		   offsetof(struct sockaddr_un, sun_path) + len + 1);
@@ -121,9 +114,8 @@ int lxc_abstract_unix_connect(const char *path)
 		errno = ENAMETOOLONG;
 		return -1;
 	}
-
-	/* do not enforce \0-termination */
-	memcpy(&addr.sun_path[1], &path[1], len);
+	/* addr.sun_path[0] has already been set to 0 by memset() */
+	strncpy(&addr.sun_path[1], &path[1], strlen(&path[1]));
 
 	ret = connect(fd, (struct sockaddr *)&addr,
 		      offsetof(struct sockaddr_un, sun_path) + len + 1);
@@ -224,7 +216,7 @@ int lxc_abstract_unix_send_credential(int fd, void *data, size_t size)
 	struct iovec iov;
 	struct cmsghdr *cmsg;
 	struct ucred cred = {
-	    .pid = lxc_raw_getpid(), .uid = getuid(), .gid = getgid(),
+	    .pid = getpid(), .uid = getuid(), .gid = getgid(),
 	};
 	char cmsgbuf[CMSG_SPACE(sizeof(cred))] = {0};
 	char buf[1] = {0};
