@@ -25,23 +25,16 @@
 #define __LXC_COMMANDS_H
 
 #include <stdio.h>
-#include <sys/types.h>
 #include <unistd.h>
+#include <sys/types.h>
 
-#include "lxccontainer.h"
-#include "macro.h"
 #include "state.h"
 
-/* Length of abstract unix domain socket socket address. */
-#define LXC_AUDS_ADDR_LEN sizeof(((struct sockaddr_un *)0)->sun_path)
-
-/* pointer conversion macros */
-#define PTR_TO_INT(p) ((int)((intptr_t)(p)))
-#define INT_TO_PTR(u) ((void *)((intptr_t)(u)))
-
-#define PTR_TO_INTMAX(p) ((intmax_t)((intptr_t)(p)))
-#define INTMAX_TO_PTR(u) ((void *)((intptr_t)(u)))
+#define LXC_CMD_DATA_MAX (MAXPATHLEN * 2)
 
+/* https://developer.gnome.org/glib/2.28/glib-Type-Conversion-Macros.html */
+#define INT_TO_PTR(n) ((void *)(long)(n))
+#define PTR_TO_INT(p) ((int)(long)(p))
 
 typedef enum {
 	LXC_CMD_CONSOLE,
@@ -55,8 +48,6 @@ typedef enum {
 	LXC_CMD_GET_NAME,
 	LXC_CMD_GET_LXCPATH,
 	LXC_CMD_ADD_STATE_CLIENT,
-	LXC_CMD_CONSOLE_LOG,
-	LXC_CMD_SERVE_STATE_CLIENTS,
 	LXC_CMD_MAX,
 } lxc_cmd_t;
 
@@ -115,13 +106,12 @@ extern int lxc_cmd_stop(const char *name, const char *lxcpath);
 extern int lxc_cmd_add_state_client(const char *name, const char *lxcpath,
 				    lxc_state_t states[MAX_STATE],
 				    int *state_client_fd);
-extern int lxc_cmd_serve_state_clients(const char *name, const char *lxcpath,
-				       lxc_state_t state);
 
 struct lxc_epoll_descr;
 struct lxc_handler;
 
-extern int lxc_cmd_init(const char *name, const char *lxcpath, const char *suffix);
+extern int lxc_cmd_init(const char *name, struct lxc_handler *handler,
+			    const char *lxcpath);
 extern int lxc_cmd_mainloop_add(const char *name, struct lxc_epoll_descr *descr,
 				    struct lxc_handler *handler);
 extern int lxc_try_cmd(const char *name, const char *lxcpath);
