@@ -51,10 +51,6 @@
 #include <mntent.h>
 #endif
 
-#ifndef HAVE_STRLCPY
-#include "include/strlcpy.h"
-#endif
-
 #define CRIU_VERSION		"2.0"
 
 #define CRIU_GITID_VERSION	"2.0"
@@ -87,7 +83,7 @@ struct criu_opts {
 	/* The path that is bind mounted from /dev/console, if any. We don't
 	 * want to use `--ext-mount-map auto`'s result here because the pts
 	 * device may have a different path (e.g. if the pty number is
-	 * different) on the target host. NULL if lxc.console = "none".
+	 * different) on the target host. NULL if lxc.console.path = "none".
 	 */
 	char *console_name;
 
@@ -110,8 +106,8 @@ static int load_tty_major_minor(char *directory, char *output, int len)
 	f = fopen(path, "r");
 	if (!f) {
 		/* This means we're coming from a liblxc which didn't export
-		 * the tty info. In this case they had to have lxc.console =
-		 * none, so there's no problem restoring.
+		 * the tty info. In this case they had to have lxc.console.path
+		 * = * none, so there's no problem restoring.
 		 */
 		if (errno == ENOENT)
 			return 0;
@@ -457,6 +453,7 @@ static void exec_criu(struct criu_opts *opts)
 		if (opts->user->predump_dir) {
 			DECLARE_ARG("--prev-images-dir");
 			DECLARE_ARG(opts->user->predump_dir);
+			DECLARE_ARG("--track-mem");
 		}
 
 		if (opts->user->pageserver_address && opts->user->pageserver_port) {
@@ -498,7 +495,7 @@ static void exec_criu(struct criu_opts *opts)
 
 		if (tty_info[0]) {
 			if (opts->console_fd < 0) {
-				ERROR("lxc.console configured on source host but not target");
+				ERROR("lxc.console.path configured on source host but not target");
 				goto err;
 			}
 
@@ -539,7 +536,6 @@ static void exec_criu(struct criu_opts *opts)
 		argv = m;
 
 		lxc_list_for_each(it, &opts->c->lxc_conf->network) {
-			size_t retlen;
 			char eth[128], *veth;
 			char *fmt;
 			struct lxc_netdev *n = it->elem;
@@ -555,10 +551,10 @@ static void exec_criu(struct criu_opts *opts)
 				external_not_veth = false;
 			}
 
-			if (n->name[0] != '\0') {
-				retlen = strlcpy(eth, n->name, sizeof(eth));
-				if (retlen >= sizeof(eth))
+			if (n->name) {
+				if (strlen(n->name) >= sizeof(eth))
 					goto err;
+				strncpy(eth, n->name, sizeof(eth));
 			} else {
 				ret = snprintf(eth, sizeof(eth), "eth%d", netnr);
 				if (ret < 0 || ret >= sizeof(eth))
@@ -568,8 +564,6 @@ static void exec_criu(struct criu_opts *opts)
 			switch (n->type) {
 			case LXC_NET_VETH:
 				veth = n->priv.veth_attr.pair;
-				if (veth[0] == '\0')
-					veth = n->priv.veth_attr.veth1;
 
 				if (n->link[0] != '\0') {
 					if (external_not_veth)
@@ -691,14 +685,14 @@ static bool criu_version_ok(char **version)
 
 		close(STDERR_FILENO);
 		if (dup2(pipes[1], STDOUT_FILENO) < 0)
-			_exit(EXIT_FAILURE);
+			exit(1);
 
 		path = on_path("criu", NULL);
 		if (!path)
-			_exit(EXIT_FAILURE);
+			exit(1);
 
 		execv(path, args);
-		_exit(EXIT_FAILURE);
+		exit(1);
 	} else {
 		FILE *f;
 		char *tmp;
@@ -769,14 +763,14 @@ static bool criu_ok(struct lxc_container *c, char **criu_version)
 {
 	struct lxc_list *it;
 
+	if (!criu_version_ok(criu_version))
+		return false;
+
 	if (geteuid()) {
 		ERROR("Must be root to checkpoint");
 		return false;
 	}
 
-	if (!criu_version_ok(criu_version))
-		return false;
-
 	/* We only know how to restore containers with veth networks. */
 	lxc_list_for_each(it, &c->lxc_conf->network) {
 		struct lxc_netdev *n = it->elem;
@@ -788,10 +782,6 @@ static bool criu_ok(struct lxc_container *c, char **criu_version)
 			break;
 		default:
 			ERROR("Found un-dumpable network: %s (%s)", lxc_net_type_to_str(n->type), n->name);
-			if (criu_version) {
-				free(*criu_version);
-				*criu_version = NULL;
-			}
 			return false;
 		}
 	}
@@ -801,7 +791,6 @@ static bool criu_ok(struct lxc_container *c, char **criu_version)
 
 static bool restore_net_info(struct lxc_container *c)
 {
-	int ret;
 	struct lxc_list *it;
 	bool has_error = true;
 
@@ -815,16 +804,14 @@ static bool restore_net_info(struct lxc_container *c)
 		if (netdev->type != LXC_NET_VETH)
 			continue;
 
-		ret = snprintf(template, sizeof(template), "vethXXXXXX");
-		if (ret < 0 || ret >= sizeof(template))
-			goto out_unlock;
+		snprintf(template, sizeof(template), "vethXXXXXX");
 
 		if (netdev->priv.veth_attr.pair[0] == '\0' &&
 		    netdev->priv.veth_attr.veth1[0] == '\0') {
 			if (!lxc_mkifname(template))
 				goto out_unlock;
 
-			(void)strlcpy(netdev->priv.veth_attr.veth1, template, IFNAMSIZ);
+			strcpy(netdev->priv.veth_attr.veth1, template);
 		}
 	}
 
@@ -836,14 +823,13 @@ out_unlock:
 }
 
 /* do_restore never returns, the calling process is used as the monitor process.
- * do_restore calls _exit() if it fails.
+ * do_restore calls exit() if it fails.
  */
 static void do_restore(struct lxc_container *c, int status_pipe, struct migrate_opts *opts, char *criu_version)
 {
-	int fd, ret;
 	pid_t pid;
 	struct lxc_handler *handler;
-	int status = 0;
+	int status, fd;
 	int pipes[2] = {-1, -1};
 
 	/* Try to detach from the current controlling tty if it exists.
@@ -881,13 +867,9 @@ static void do_restore(struct lxc_container *c, int status_pipe, struct migrate_
 		goto out_fini_handler;
 	}
 
-	ret = resolve_clone_flags(handler);
-	if (ret < 0) {
-		ERROR("%s - Unsupported clone flag specified", strerror(errno));
-		goto out_fini_handler;
-	}
+	resolve_clone_flags(handler);
 
-	if (pipe2(pipes, O_CLOEXEC) < 0) {
+	if (pipe(pipes) < 0) {
 		SYSERROR("pipe() failed");
 		goto out_fini_handler;
 	}
@@ -983,7 +965,7 @@ static void do_restore(struct lxc_container *c, int status_pipe, struct migrate_
 			if (WEXITSTATUS(status)) {
 				int n;
 
-				n = lxc_read_nointr(pipes[0], buf, sizeof(buf));
+				n = read(pipes[0], buf, sizeof(buf));
 				if (n < 0) {
 					SYSERROR("failed reading from criu stderr");
 					goto out_fini_handler;
@@ -1027,7 +1009,7 @@ static void do_restore(struct lxc_container *c, int status_pipe, struct migrate_
 
 		close(pipes[0]);
 
-		ret = lxc_write_nointr(status_pipe, &status, sizeof(status));
+		ret = write(status_pipe, &status, sizeof(status));
 		close(status_pipe);
 		status_pipe = -1;
 
@@ -1042,18 +1024,13 @@ static void do_restore(struct lxc_container *c, int status_pipe, struct migrate_
 		 * assign the return here to silence potential.
 		 */
 		ret = snprintf(title, sizeof(title), "[lxc monitor] %s %s", c->config_path, c->name);
-		if (ret < 0 || (size_t)ret >= sizeof(title))
-			INFO("Setting truncated process name");
-
 		ret = setproctitle(title);
-		if (ret < 0)
-			INFO("Failed to set process name");
 
 		ret = lxc_poll(c->name, handler);
 		if (ret)
 			lxc_abort(c->name, handler);
 		lxc_fini(c->name, handler);
-		_exit(ret);
+		exit(ret);
 	}
 
 out_fini_handler:
@@ -1072,13 +1049,13 @@ out:
 		 */
 		if (!status)
 			status = 1;
-
-		if (lxc_write_nointr(status_pipe, &status, sizeof(status)) != sizeof(status))
+		if (write(status_pipe, &status, sizeof(status)) != sizeof(status)) {
 			SYSERROR("writing status failed");
+		}
 		close(status_pipe);
 	}
 
-	_exit(EXIT_FAILURE);
+	exit(1);
 }
 
 static int save_tty_major_minor(char *directory, struct lxc_container *c, char *tty_id, int len)
@@ -1135,18 +1112,15 @@ static int save_tty_major_minor(char *directory, struct lxc_container *c, char *
 /* do one of either predump or a regular dump */
 static bool do_dump(struct lxc_container *c, char *mode, struct migrate_opts *opts)
 {
-	int ret;
 	pid_t pid;
-	int criuout[2];
 	char *criu_version = NULL;
+	int criuout[2];
 
 	if (!criu_ok(c, &criu_version))
 		return false;
 
-	ret = pipe(criuout);
-	if (ret < 0) {
+	if (pipe(criuout) < 0) {
 		SYSERROR("pipe() failed");
-		free(criu_version);
 		return false;
 	}
 
@@ -1165,12 +1139,10 @@ static bool do_dump(struct lxc_container *c, char *mode, struct migrate_opts *op
 
 		close(criuout[0]);
 
-		lxc_zero_handler(&h);
-
 		h.name = c->name;
 		if (!cgroup_init(&h)) {
 			ERROR("failed to cgroup_init()");
-			_exit(EXIT_FAILURE);
+			exit(1);
 		}
 
 		os.pipefd = criuout[1];
@@ -1180,16 +1152,12 @@ static bool do_dump(struct lxc_container *c, char *mode, struct migrate_opts *op
 		os.console_name = c->lxc_conf->console.path;
 		os.criu_version = criu_version;
 
-		ret = save_tty_major_minor(opts->directory, c, os.tty_id, sizeof(os.tty_id));
-		if (ret < 0) {
-			free(criu_version);
-			_exit(EXIT_FAILURE);
-		}
+		if (save_tty_major_minor(opts->directory, c, os.tty_id, sizeof(os.tty_id)) < 0)
+			exit(1);
 
 		/* exec_criu() returning is an error */
 		exec_criu(&os);
-		free(criu_version);
-		_exit(EXIT_FAILURE);
+		exit(1);
 	} else {
 		int status;
 		ssize_t n;
@@ -1202,21 +1170,16 @@ static bool do_dump(struct lxc_container *c, char *mode, struct migrate_opts *op
 		if (w == -1) {
 			SYSERROR("waitpid");
 			close(criuout[0]);
-			free(criu_version);
 			return false;
 		}
 
-		n = lxc_read_nointr(criuout[0], buf, sizeof(buf));
+		n = read(criuout[0], buf, sizeof(buf));
 		close(criuout[0]);
 		if (n < 0) {
 			SYSERROR("read");
 			n = 0;
 		}
-
-		if (n == sizeof(buf))
-			buf[n-1] = 0;
-		else
-			buf[n] = 0;
+		buf[n] = 0;
 
 		if (WIFEXITED(status)) {
 			if (WEXITSTATUS(status)) {
@@ -1235,15 +1198,12 @@ static bool do_dump(struct lxc_container *c, char *mode, struct migrate_opts *op
 
 		if (!ret)
 			ERROR("criu output: %s", buf);
-
-		free(criu_version);
 		return ret;
 	}
 fail:
 	close(criuout[0]);
 	close(criuout[1]);
 	rmdir(opts->directory);
-	free(criu_version);
 	return false;
 }
 
@@ -1276,6 +1236,9 @@ bool __criu_restore(struct lxc_container *c, struct migrate_opts *opts)
 	int pipefd[2];
 	char *criu_version = NULL;
 
+	if (!criu_ok(c, &criu_version))
+		return false;
+
 	if (geteuid()) {
 		ERROR("Must be root to restore");
 		return false;
@@ -1286,39 +1249,32 @@ bool __criu_restore(struct lxc_container *c, struct migrate_opts *opts)
 		return false;
 	}
 
-	if (!criu_ok(c, &criu_version)) {
-		close(pipefd[0]);
-		close(pipefd[1]);
-		return false;
-	}
-
 	pid = fork();
 	if (pid < 0) {
 		close(pipefd[0]);
 		close(pipefd[1]);
-		free(criu_version);
 		return false;
 	}
 
 	if (pid == 0) {
 		close(pipefd[0]);
-		// this never returns
+		/* this never returns */
 		do_restore(c, pipefd[1], opts, criu_version);
 	}
 
 	close(pipefd[1]);
-	free(criu_version);
 
-	nread = lxc_read_nointr(pipefd[0], &status, sizeof(status));
+	nread = read(pipefd[0], &status, sizeof(status));
 	close(pipefd[0]);
 	if (sizeof(status) != nread) {
 		ERROR("reading status from pipe failed");
 		goto err_wait;
 	}
 
-	// If the criu process was killed or exited nonzero, wait() for the
-	// handler, since the restore process died. Otherwise, we don't need to
-	// wait, since the child becomes the monitor process.
+	/* If the criu process was killed or exited nonzero, wait() for the
+	 * handler, since the restore process died. Otherwise, we don't need to
+	 * wait, since the child becomes the monitor process.
+	 */
 	if (!WIFEXITED(status) || WEXITSTATUS(status))
 		goto err_wait;
 	return true;
