@@ -35,6 +35,9 @@
 #include <pthread.h>
 #include <time.h>
 
+#include <syslog.h>
+#include <stdio.h>
+
 #include <fcntl.h>
 #include <stdlib.h>
 
@@ -50,15 +53,79 @@
 #define LXC_LOG_TIME_SIZE ((LXC_NUMSTRLEN64)*2)
 
 int lxc_log_fd = -1;
+static int syslog_enable = 0;
 int lxc_quiet_specified;
 int lxc_log_use_global_fd;
 static int lxc_loglevel_specified;
 
 static char log_prefix[LXC_LOG_PREFIX_SIZE] = "lxc";
 static char *log_fname = NULL;
+static char *log_vmname = NULL;
 
 lxc_log_define(lxc_log, lxc);
 
+static int lxc_log_priority_to_syslog(int priority)
+{
+	switch (priority) {
+	case LXC_LOG_LEVEL_FATAL:
+		return LOG_EMERG;
+	case LXC_LOG_LEVEL_ALERT:
+		return LOG_ALERT;
+	case LXC_LOG_LEVEL_CRIT:
+		return LOG_CRIT;
+	case LXC_LOG_LEVEL_ERROR:
+		return LOG_ERR;
+	case LXC_LOG_LEVEL_WARN:
+		return LOG_WARNING;
+	case LXC_LOG_LEVEL_NOTICE:
+	case LXC_LOG_LEVEL_NOTSET:
+		return LOG_NOTICE;
+	case LXC_LOG_LEVEL_INFO:
+		return LOG_INFO;
+	case LXC_LOG_LEVEL_TRACE:
+	case LXC_LOG_LEVEL_DEBUG:
+		return LOG_DEBUG;
+	}
+
+	/* Not reached */
+	return LOG_NOTICE;
+}
+
+/*---------------------------------------------------------------------------*/
+static int log_append_syslog(const struct lxc_log_appender *appender,
+			     struct lxc_log_event *event)
+{
+	char *msg;
+	int rc, len;
+	va_list args;
+
+	if (!syslog_enable)
+		return 0;
+
+	va_copy(args, *event->vap);
+	len = vsnprintf(NULL, 0, event->fmt, args) + 1;
+	va_end(args);
+	msg = malloc(len * sizeof(char));
+	if (msg == NULL)
+		return 0;
+	rc = vsnprintf(msg, len, event->fmt, *event->vap);
+	if (rc == -1 || rc >= len) {
+		free(msg);
+		return 0;
+	}
+
+	syslog(lxc_log_priority_to_syslog(event->priority),
+		"%s%s %s - %s:%s:%d - %s" ,
+		log_vmname ? log_vmname : "",
+		log_vmname ? ":" : "",
+		event->category,
+		event->locinfo->file, event->locinfo->func,
+		event->locinfo->line,
+		msg);
+	free(msg);
+	return 0;
+}
+
 /*---------------------------------------------------------------------------*/
 static int log_append_stderr(const struct lxc_log_appender *appender,
 			     struct lxc_log_event *event)
@@ -66,7 +133,7 @@ static int log_append_stderr(const struct lxc_log_appender *appender,
 	if (event->priority < LXC_LOG_LEVEL_ERROR)
 		return 0;
 
-	fprintf(stderr, "%s: ", log_prefix);
+	fprintf(stderr, "%s: %s%s", log_prefix, log_vmname ? log_vmname : "", log_vmname ? ": " : "");
 	fprintf(stderr, "%s: %s: %d ", event->locinfo->file, event->locinfo->func, event->locinfo->line);
 	vfprintf(stderr, event->fmt, *event->vap);
 	fprintf(stderr, "\n");
@@ -155,7 +222,7 @@ int lxc_unix_epoch_to_utc(char *buf, size_t bufsize, const struct timespec *time
 	seconds = (time->tv_sec - d_in_s - h_in_s - (minutes * 60));
 
 	/* Make string from nanoseconds. */
-	ret = snprintf(nanosec, LXC_NUMSTRLEN64, "%"PRId64, (int64_t)time->tv_nsec);
+	ret = snprintf(nanosec, LXC_NUMSTRLEN64, "%ld", time->tv_nsec);
 	if (ret < 0 || ret >= LXC_NUMSTRLEN64)
 		return -1;
 
@@ -215,8 +282,10 @@ static int log_append_logfile(const struct lxc_log_appender *appender,
 		return 0;
 
 	n = snprintf(buffer, sizeof(buffer),
-			"%15s %s %-8s %s - %s:%s:%d - ",
+			"%15s%s%s %s %-8s %s - %s:%s:%d - ",
 			log_prefix,
+			log_vmname ? " " : "",
+			log_vmname ? log_vmname : "",
 			date_time,
 			lxc_log_priority_to_string(event->priority),
 			event->category,
@@ -228,8 +297,7 @@ static int log_append_logfile(const struct lxc_log_appender *appender,
 
 	if ((size_t)n < (sizeof(buffer) - 1))
 		n += vsnprintf(buffer + n, sizeof(buffer) - n, event->fmt, *event->vap);
-
-	if ((size_t)n >= sizeof(buffer))
+	else
 		n = sizeof(buffer) - 1;
 
 	buffer[n] = '\n';
@@ -237,6 +305,12 @@ static int log_append_logfile(const struct lxc_log_appender *appender,
 	return write(fd_to_use, buffer, n + 1);
 }
 
+static struct lxc_log_appender log_appender_syslog = {
+	.name		= "syslog",
+	.append		= log_append_syslog,
+	.next		= NULL,
+};
+
 static struct lxc_log_appender log_appender_stderr = {
 	.name		= "stderr",
 	.append		= log_append_stderr,
@@ -266,10 +340,11 @@ struct lxc_log_category lxc_log_category_lxc = {
 /*---------------------------------------------------------------------------*/
 static int build_dir(const char *name)
 {
-	char *n = strdup(name);  // because we'll be modifying it
-	char *p, *e;
 	int ret;
+	char *e, *n, *p;
 
+	/* Make copy of string since we'll be modifying it. */
+	n = strdup(name);
 	if (!n) {
 		ERROR("Out of memory while creating directory '%s'.", name);
 		return -1;
@@ -375,6 +450,9 @@ static char *build_log_path(const char *name, const char *lxcpath)
 
 extern void lxc_log_close(void)
 {
+	closelog();
+	free(log_vmname);
+	log_vmname = NULL;
 	if (lxc_log_fd == -1)
 		return;
 	close(lxc_log_fd);
@@ -386,17 +464,16 @@ extern void lxc_log_close(void)
 /*
  * This can be called:
  *   1. when a program calls lxc_log_init with no logfile parameter (in which
- *      case the default is used).  In this case lxc.logfile can override this.
+ *      case the default is used).  In this case lxc.loge can override this.
  *   2. when a program calls lxc_log_init with a logfile parameter.  In this
- *	case we don't want lxc.logfile to override this.
- *   3. When a lxc.logfile entry is found in config file.
+ *	case we don't want lxc.log to override this.
+ *   3. When a lxc.log entry is found in config file.
  */
 static int __lxc_log_set_file(const char *fname, int create_dirs)
 {
-	if (lxc_log_fd != -1) {
-		// we are overriding the default.
+	/* we are overriding the default. */
+	if (lxc_log_fd != -1)
 		lxc_log_close();
-	}
 
 	if (!fname)
 		return -1;
@@ -407,8 +484,9 @@ static int __lxc_log_set_file(const char *fname, int create_dirs)
 	}
 
 #if USE_CONFIGPATH_LOGS
-	// we don't build_dir for the default if the default is
-	// i.e. /var/lib/lxc/$container/$container.log
+	/* We don't build_dir for the default if the default is i.e.
+	 * /var/lib/lxc/$container/$container.log.
+	 */
 	if (create_dirs)
 #endif
 	if (build_dir(fname)) {
@@ -440,6 +518,28 @@ static int _lxc_log_set_file(const char *name, const char *lxcpath, int create_d
 	return ret;
 }
 
+extern int lxc_log_syslog(int facility)
+{
+	struct lxc_log_appender *appender;
+
+	openlog(log_prefix, LOG_PID, facility);
+	if (!lxc_log_category_lxc.appender) {
+		lxc_log_category_lxc.appender = &log_appender_syslog;
+		return 0;
+	}
+	appender = lxc_log_category_lxc.appender;
+	while (appender->next != NULL)
+		appender = appender->next;
+	appender->next = &log_appender_syslog;
+
+	return 0;
+}
+
+extern void lxc_log_enable_syslog(void)
+{
+	syslog_enable = 1;
+}
+
 /*
  * lxc_log_init:
  * Called from lxc front-end programs (like lxc-create, lxc-start) to
@@ -471,6 +571,9 @@ extern int lxc_log_init(struct lxc_log *log)
 	if (log->prefix)
 		lxc_log_set_prefix(log->prefix);
 
+	if (log->name)
+		log_vmname = strdup(log->name);
+
 	if (log->file) {
 		if (strcmp(log->file, "none") == 0)
 			return 0;
@@ -512,7 +615,7 @@ extern int lxc_log_init(struct lxc_log *log)
 }
 
 /*
- * This is called when we read a lxc.loglevel entry in a lxc.conf file.  This
+ * This is called when we read a lxc.log.level entry in a lxc.conf file.  This
  * happens after processing command line arguments, which override the .conf
  * settings.  So only set the level if previously unset.
  */
