@@ -19,14 +19,13 @@
  */
 
 #define _GNU_SOURCE
-#include <errno.h>
-#include <fcntl.h>
 #include <malloc.h>
-#include <pthread.h>
 #include <stdio.h>
-#include <stdlib.h>
-#include <sys/file.h>
+#include <errno.h>
 #include <unistd.h>
+#include <fcntl.h>
+#include <stdlib.h>
+#include <pthread.h>
 
 #include <lxc/lxccontainer.h>
 
@@ -56,7 +55,7 @@ static inline void dump_stacktrace(void)
 	strings = backtrace_symbols(array, size);
 
 	/* Using fprintf here as our logging module is not thread safe. */
-	fprintf(stderr, "\tObtained %zu stack frames\n", size);
+	fprintf(stderr, "\tObtained %zu stack frames.\n", size);
 
 	for (i = 0; i < size; i++)
 		fprintf(stderr, "\t\t%s\n", strings[i]);
@@ -73,11 +72,10 @@ static void lock_mutex(pthread_mutex_t *l)
 {
 	int ret;
 
-	ret = pthread_mutex_lock(l);
-	if (ret != 0) {
-		SYSERROR("Failed to acquire mutex");
+	if ((ret = pthread_mutex_lock(l)) != 0) {
+		fprintf(stderr, "pthread_mutex_lock returned:%d %s\n", ret, strerror(ret));
 		dump_stacktrace();
-		_exit(EXIT_FAILURE);
+		exit(EXIT_FAILURE);
 	}
 }
 
@@ -85,11 +83,10 @@ static void unlock_mutex(pthread_mutex_t *l)
 {
 	int ret;
 
-	ret = pthread_mutex_unlock(l);
-	if (ret != 0) {
-		SYSERROR("Failed to release mutex");
+	if ((ret = pthread_mutex_unlock(l)) != 0) {
+		fprintf(stderr, "pthread_mutex_unlock returned:%d %s\n", ret, strerror(ret));
 		dump_stacktrace();
-		_exit(EXIT_FAILURE);
+		exit(EXIT_FAILURE);
 	}
 }
 
@@ -111,7 +108,6 @@ static char *lxclock_name(const char *p, const char *n)
 	rundir = get_rundir();
 	if (!rundir)
 		return NULL;
-
 	len += strlen(rundir);
 
 	if ((dest = malloc(len)) == NULL) {
@@ -125,7 +121,6 @@ static char *lxclock_name(const char *p, const char *n)
 		free(rundir);
 		return NULL;
 	}
-
 	ret = mkdir_p(dest, 0755);
 	if (ret < 0) {
 		free(dest);
@@ -139,7 +134,6 @@ static char *lxclock_name(const char *p, const char *n)
 		free(dest);
 		return NULL;
 	}
-
 	return dest;
 }
 
@@ -151,13 +145,11 @@ static sem_t *lxc_new_unnamed_sem(void)
 	s = malloc(sizeof(*s));
 	if (!s)
 		return NULL;
-
 	ret = sem_init(s, 0, 1);
 	if (ret) {
 		free(s);
 		return NULL;
 	}
-
 	return s;
 }
 
@@ -176,7 +168,6 @@ struct lxc_lock *lxc_newlock(const char *lxcpath, const char *name)
 			free(l);
 			l = NULL;
 		}
-
 		goto out;
 	}
 
@@ -187,7 +178,6 @@ struct lxc_lock *lxc_newlock(const char *lxcpath, const char *name)
 		l = NULL;
 		goto out;
 	}
-
 	l->u.f.fd = -1;
 
 out:
@@ -203,19 +193,17 @@ int lxclock(struct lxc_lock *l, int timeout)
 	case LXC_LOCK_ANON_SEM:
 		if (!timeout) {
 			ret = sem_wait(l->u.sem);
-			if (ret < 0)
+			if (ret == -1)
 				saved_errno = errno;
 		} else {
 			struct timespec ts;
-
 			if (clock_gettime(CLOCK_REALTIME, &ts) == -1) {
 				ret = -2;
 				goto out;
 			}
-
 			ts.tv_sec += timeout;
 			ret = sem_timedwait(l->u.sem, &ts);
-			if (ret < 0)
+			if (ret == -1)
 				saved_errno = errno;
 		}
 		break;
@@ -225,32 +213,26 @@ int lxclock(struct lxc_lock *l, int timeout)
 			ERROR("Error: timeout not supported with flock");
 			goto out;
 		}
-
 		if (!l->u.f.fname) {
 			ERROR("Error: filename not set for flock");
 			goto out;
 		}
-
 		if (l->u.f.fd == -1) {
-			l->u.f.fd = open(l->u.f.fname, O_CREAT | O_RDWR | O_NOFOLLOW | O_CLOEXEC | O_NOCTTY, S_IWUSR | S_IRUSR);
+			l->u.f.fd = open(l->u.f.fname, O_RDWR|O_CREAT,
+					S_IWUSR | S_IRUSR);
 			if (l->u.f.fd == -1) {
 				ERROR("Error opening %s", l->u.f.fname);
 				saved_errno = errno;
 				goto out;
 			}
 		}
-
-		memset(&lk, 0, sizeof(struct flock));
-
 		lk.l_type = F_WRLCK;
 		lk.l_whence = SEEK_SET;
-
-		ret = fcntl(l->u.f.fd, F_OFD_SETLKW, &lk);
-		if (ret < 0) {
-			if (errno == EINVAL)
-				ret = flock(l->u.f.fd, LOCK_EX);
+		lk.l_start = 0;
+		lk.l_len = 0;
+		ret = fcntl(l->u.f.fd, F_SETLKW, &lk);
+		if (ret == -1)
 			saved_errno = errno;
-		}
 		break;
 	}
 
@@ -275,18 +257,13 @@ int lxcunlock(struct lxc_lock *l)
 		break;
 	case LXC_LOCK_FLOCK:
 		if (l->u.f.fd != -1) {
-			memset(&lk, 0, sizeof(struct flock));
-
 			lk.l_type = F_UNLCK;
 			lk.l_whence = SEEK_SET;
-
-			ret = fcntl(l->u.f.fd, F_OFD_SETLK, &lk);
-			if (ret < 0) {
-				if (errno == EINVAL)
-					ret = flock(l->u.f.fd, LOCK_EX | LOCK_NB);
+			lk.l_start = 0;
+			lk.l_len = 0;
+			ret = fcntl(l->u.f.fd, F_SETLK, &lk);
+			if (ret < 0)
 				saved_errno = errno;
-			}
-
 			close(l->u.f.fd);
 			l->u.f.fd = -1;
 		} else
@@ -309,7 +286,6 @@ void lxc_putlock(struct lxc_lock *l)
 {
 	if (!l)
 		return;
-
 	switch(l->type) {
 	case LXC_LOCK_ANON_SEM:
 		if (l->u.sem) {
@@ -323,7 +299,6 @@ void lxc_putlock(struct lxc_lock *l)
 			close(l->u.f.fd);
 			l->u.f.fd = -1;
 		}
-
 		free(l->u.f.fname);
 		l->u.f.fname = NULL;
 		break;
@@ -341,6 +316,23 @@ void process_unlock(void)
 	unlock_mutex(&thread_mutex);
 }
 
+/* One thread can do fork() while another one is holding a mutex.
+ * There is only one thread in child just after the fork(), so no one will ever release that mutex.
+ * We setup a "child" fork handler to unlock the mutex just after the fork().
+ * For several mutex types, unlocking an unlocked mutex can lead to undefined behavior.
+ * One way to deal with it is to setup "prepare" fork handler
+ * to lock the mutex before fork() and both "parent" and "child" fork handlers
+ * to unlock the mutex.
+ * This forbids doing fork() while explicitly holding the lock.
+ */
+#ifdef HAVE_PTHREAD_ATFORK
+__attribute__((constructor))
+static void process_lock_setup_atfork(void)
+{
+	pthread_atfork(process_lock, process_unlock, process_unlock);
+}
+#endif
+
 int container_mem_lock(struct lxc_container *c)
 {
 	return lxclock(c->privlock, 0);
@@ -357,12 +349,10 @@ int container_disk_lock(struct lxc_container *c)
 
 	if ((ret = lxclock(c->privlock, 0)))
 		return ret;
-
 	if ((ret = lxclock(c->slock, 0))) {
 		lxcunlock(c->privlock);
 		return ret;
 	}
-
 	return 0;
 }
 
