@@ -32,6 +32,7 @@
 #include <unistd.h>
 #include <sys/ioctl.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <sys/types.h>
 #include <sys/vfs.h>
 
@@ -41,20 +42,8 @@
 #include "storage.h"
 #include "utils.h"
 
-#ifndef HAVE_STRLCAT
-#include "include/strlcat.h"
-#endif
-
-#ifndef HAVE_STRLCPY
-#include "include/strlcpy.h"
-#endif
-
 lxc_log_define(btrfs, lxc);
 
-/* defined in lxccontainer.c: needs to become common helper */
-extern char *dir_new_path(char *src, const char *oldname, const char *name,
-			  const char *oldpath, const char *lxcpath);
-
 /*
  * Return the full path of objid under dirid.  Let's say dirid is
  * /lxc/c1/rootfs, and objid is /lxc/c1/rootfs/a/b/c.  Then we will
@@ -62,11 +51,11 @@ extern char *dir_new_path(char *src, const char *oldname, const char *name,
  * simply return a.
  */
 char *get_btrfs_subvol_path(int fd, u64 dir_id, u64 objid, char *name,
-			    u16 name_len)
+			    int name_len)
 {
 	struct btrfs_ioctl_ino_lookup_args args;
-	int ret;
-	size_t len, retlen;
+	int ret, e;
+	size_t len;
 	char *retpath;
 
 	memset(&args, 0, sizeof(args));
@@ -74,16 +63,17 @@ char *get_btrfs_subvol_path(int fd, u64 dir_id, u64 objid, char *name,
 	args.objectid = objid;
 
 	ret = ioctl(fd, BTRFS_IOC_INO_LOOKUP, &args);
+	e = errno;
 	if (ret) {
-		SYSERROR("Failed to lookup path for %llu %llu %s",
-		         (unsigned long long) dir_id,
-		         (unsigned long long) objid,
-		         name);
+		ERROR("Failed to lookup path for %llu %llu %s - %s\n",
+				 (unsigned long long) dir_id,
+				 (unsigned long long) objid,
+				 name, strerror(e));
 		return NULL;
 	} else
-		INFO("Got path for %llu %llu - %s",
-		     (unsigned long long) objid, (unsigned long long) dir_id,
-		     name);
+		INFO("Got path for %llu %llu - %s\n",
+			(unsigned long long) objid, (unsigned long long) dir_id,
+			name);
 
 	if (args.name[0]) {
 		/*
@@ -94,40 +84,21 @@ char *get_btrfs_subvol_path(int fd, u64 dir_id, u64 objid, char *name,
 		retpath = malloc(len);
 		if (!retpath)
 			return NULL;
-
-		(void)strlcpy(retpath, args.name, len);
-		(void)strlcat(retpath, "/", len);
-
-		retlen = strlcat(retpath, name, len);
-		if (retlen >= len) {
-			ERROR("Failed to append name - %s", name);
-			free(retpath);
-			return NULL;
-		}
+		strcpy(retpath, args.name);
+		strcat(retpath, "/");
+		strncat(retpath, name, name_len);
 	} else {
 		/* we're at the root of ref_tree */
 		len = name_len + 1;
 		retpath = malloc(len);
 		if (!retpath)
 			return NULL;
-
 		*retpath = '\0';
-
-		retlen = strlcat(retpath, name, len);
-		if (retlen >= len) {
-			ERROR("Failed to append name - %s", name);
-			free(retpath);
-			return NULL;
-		}
+		strncat(retpath, name, name_len);
 	}
-
 	return retpath;
 }
 
-//
-// btrfs ops
-//
-
 int btrfs_list_get_path_rootid(int fd, u64 *treeid)
 {
 	int  ret;
@@ -151,7 +122,7 @@ bool is_btrfs_fs(const char *path)
 	int fd, ret;
 	struct btrfs_ioctl_space_args sargs;
 
-	// make sure this is a btrfs filesystem
+	/* Make sure this is a btrfs filesystem. */
 	fd = open(path, O_RDONLY);
 	if (fd < 0)
 		return false;
@@ -191,33 +162,37 @@ int is_btrfs_subvol(const char *path)
 	return stfs.f_type == BTRFS_SUPER_MAGIC;
 }
 
-int btrfs_detect(const char *path)
+bool btrfs_detect(const char *path)
 {
 	struct stat st;
 	int ret;
 
-	if (!is_btrfs_fs(path))
-		return 0;
+	if (!strncmp(path, "btrfs:", 6))
+		return true;
 
-	// and make sure it's a subvolume.
+	if (!is_btrfs_fs(path))
+		return false;
+
+	/* make sure it's a subvolume */
 	ret = stat(path, &st);
 	if (ret < 0)
-		return 0;
+		return false;
 
 	if (st.st_ino == 256 && S_ISDIR(st.st_mode))
-		return 1;
+		return true;
 
-	return 0;
+	return false;
 }
 
 int btrfs_mount(struct lxc_storage *bdev)
 {
 	unsigned long mntflags;
-	char *mntdata;
+	char *mntdata, *src;
 	int ret;
 
 	if (strcmp(bdev->type, "btrfs"))
 		return -22;
+
 	if (!bdev->src || !bdev->dest)
 		return -22;
 
@@ -226,7 +201,9 @@ int btrfs_mount(struct lxc_storage *bdev)
 		return -22;
 	}
 
-	ret = mount(bdev->src, bdev->dest, "bind", MS_BIND | MS_REC | mntflags, mntdata);
+	src = lxc_storage_get_path(bdev->src, "btrfs");
+
+	ret = mount(src, bdev->dest, "bind", MS_BIND | MS_REC | mntflags, mntdata);
 	free(mntdata);
 	return ret;
 }
@@ -235,15 +212,16 @@ int btrfs_umount(struct lxc_storage *bdev)
 {
 	if (strcmp(bdev->type, "btrfs"))
 		return -22;
+
 	if (!bdev->src || !bdev->dest)
 		return -22;
+
 	return umount(bdev->dest);
 }
 
 static int btrfs_subvolume_create(const char *path)
 {
 	int ret, saved_errno;
-	size_t retlen;
 	struct btrfs_ioctl_vol_args args;
 	char *p, *newfull;
 	int fd = -1;
@@ -269,12 +247,8 @@ static int btrfs_subvolume_create(const char *path)
 	}
 
 	memset(&args, 0, sizeof(args));
-	retlen = strlcpy(args.name, p + 1, BTRFS_SUBVOL_NAME_MAX);
-	if (retlen >= BTRFS_SUBVOL_NAME_MAX) {
-		free(newfull);
-		close(fd);
-		return -E2BIG;
-	}
+	strncpy(args.name, p + 1, BTRFS_SUBVOL_NAME_MAX);
+	args.name[BTRFS_SUBVOL_NAME_MAX - 1] = 0;
 
 	ret = ioctl(fd, BTRFS_IOC_SUBVOL_CREATE, &args);
 	saved_errno = errno;
@@ -328,7 +302,6 @@ out:
 
 int btrfs_snapshot(const char *orig, const char *new)
 {
-	size_t retlen;
 	struct btrfs_ioctl_vol_args_v2 args;
 	char *newdir, *newname;
 	char *newfull = NULL;
@@ -355,9 +328,8 @@ int btrfs_snapshot(const char *orig, const char *new)
 
 	memset(&args, 0, sizeof(args));
 	args.fd = fd;
-	retlen = strlcpy(args.name, newname, BTRFS_SUBVOL_NAME_MAX);
-	if (retlen >= BTRFS_SUBVOL_NAME_MAX)
-		goto out;
+	strncpy(args.name, newname, BTRFS_SUBVOL_NAME_MAX);
+	args.name[BTRFS_SUBVOL_NAME_MAX - 1] = 0;
 
 	ret = ioctl(fddst, BTRFS_IOC_SNAP_CREATE_V2, &args);
 	saved_errno = errno;
@@ -365,32 +337,34 @@ int btrfs_snapshot(const char *orig, const char *new)
 out:
 	if (fddst != -1)
 		close(fddst);
-
 	if (fd != -1)
 		close(fd);
-
 	free(newfull);
 
 	if (saved_errno >= 0)
 		errno = saved_errno;
-
 	return ret;
 }
 
 int btrfs_snapshot_wrapper(void *data)
 {
+	char *src;
 	struct rsync_data_char *arg = data;
+
 	if (setgid(0) < 0) {
 		ERROR("Failed to setgid to 0");
 		return -1;
 	}
 	if (setgroups(0, NULL) < 0)
 		WARN("Failed to clear groups");
+
 	if (setuid(0) < 0) {
 		ERROR("Failed to setuid to 0");
 		return -1;
 	}
-	return btrfs_snapshot(arg->src, arg->dest);
+
+	src = lxc_storage_get_path(arg->src, "btrfs");
+	return btrfs_snapshot(src, arg->dest);
 }
 
 int btrfs_clonepaths(struct lxc_storage *orig, struct lxc_storage *new,
@@ -398,57 +372,131 @@ int btrfs_clonepaths(struct lxc_storage *orig, struct lxc_storage *new,
 		     const char *oldpath, const char *lxcpath, int snap,
 		     uint64_t newsize, struct lxc_conf *conf)
 {
+	char *src;
+
 	if (!orig->dest || !orig->src)
 		return -1;
 
-	if (strcmp(orig->type, "btrfs")) {
-		int len, ret;
-		if (snap) {
-			ERROR("btrfs snapshot from %s backing store is not supported",
-				orig->type);
+	if (strcmp(orig->type, "btrfs") && snap) {
+		ERROR("btrfs snapshot from %s backing store is not supported",
+		      orig->type);
+		return -1;
+	}
+
+	new->src = lxc_string_join(
+	    "/",
+	    (const char *[]){"btrfs:", *lxcpath != '/' ? lxcpath : ++lxcpath,
+			     cname, "rootfs", NULL},
+	    false);
+	if (!new->src) {
+		ERROR("Failed to create new rootfs path");
+		return -1;
+	}
+	TRACE("Constructed new rootfs path \"%s\"", new->src);
+
+	src = lxc_storage_get_path(new->src, "btrfs");
+	new->dest = strdup(src);
+	if (!new->dest) {
+		ERROR("Failed to duplicate string \"%s\"", src);
+		return -1;
+	}
+
+	if (orig->mntopts) {
+		new->mntopts = strdup(orig->mntopts);
+		if (!new->mntopts) {
+			ERROR("Failed to duplicate string \"%s\"",
+			      orig->mntopts);
 			return -1;
 		}
-		len = strlen(lxcpath) + strlen(cname) + strlen("rootfs") + 3;
-		new->src = malloc(len);
-		if (!new->src)
-			return -1;
-		ret = snprintf(new->src, len, "%s/%s/rootfs", lxcpath, cname);
-		if (ret < 0 || ret >= len)
-			return -1;
-	} else {
-		// in case rootfs is in custom path, reuse it
-		if ((new->src = dir_new_path(orig->src, oldname, cname, oldpath, lxcpath)) == NULL)
-			return -1;
 	}
 
-	if ((new->dest = strdup(new->src)) == NULL)
-		return -1;
+	return 0;
+}
 
-	if (orig->mntopts && (new->mntopts = strdup(orig->mntopts)) == NULL)
-		return -1;
+bool btrfs_create_clone(struct lxc_conf *conf, struct lxc_storage *orig,
+			struct lxc_storage *new, uint64_t newsize)
+{
+	int ret;
+	struct rsync_data data = {0, 0};
+	char cmd_output[MAXPATHLEN] = {0};
 
-	if (snap) {
-		struct rsync_data_char sdata;
-		if (!am_guest_unpriv())
-			return btrfs_snapshot(orig->dest, new->dest);
-		sdata.dest = new->dest;
-		sdata.src = orig->dest;
-		return userns_exec_full(conf, btrfs_snapshot_wrapper, &sdata,
-					"btrfs_snapshot_wrapper");
+	ret = rmdir(new->dest);
+	if (ret < 0 && errno != ENOENT)
+		return false;
+
+	ret = btrfs_subvolume_create(new->dest);
+	if (ret < 0) {
+		SYSERROR("Failed to create btrfs subvolume \"%s\"", new->dest);
+		return false;
 	}
 
-	if (rmdir(new->dest) < 0 && errno != ENOENT) {
-		SYSERROR("removing %s", new->dest);
-		return -1;
+	/* rsync the contents from source to target */
+	data.orig = orig;
+	data.new = new;
+	if (am_unpriv()) {
+		ret = userns_exec_1(conf, lxc_storage_rsync_exec_wrapper, &data,
+				    "lxc_storage_rsync_exec_wrapper");
+		if (ret < 0) {
+			ERROR("Failed to rsync from \"%s\" into \"%s\"",
+			      orig->dest, new->dest);
+			return false;
+		}
+
+		return true;
 	}
 
-	return btrfs_subvolume_create(new->dest);
+	ret = run_command(cmd_output, sizeof(cmd_output),
+			lxc_storage_rsync_exec_wrapper, (void *)&data);
+	if (ret < 0) {
+		ERROR("Failed to rsync from \"%s\" into \"%s\": %s", orig->dest,
+		      new->dest, cmd_output);
+		return false;
+	}
+
+	return true;
+}
+
+bool btrfs_create_snapshot(struct lxc_conf *conf, struct lxc_storage *orig,
+			   struct lxc_storage *new, uint64_t newsize)
+{
+	int ret;
+
+	ret = rmdir(new->dest);
+	if (ret < 0 && errno != ENOENT)
+		return false;
+
+	if (am_unpriv()) {
+		struct rsync_data_char args;
+
+		args.src = orig->dest;
+		args.dest = new->dest;
+
+		ret = userns_exec_1(conf, btrfs_snapshot_wrapper, &args,
+				"btrfs_snapshot_wrapper");
+		if (ret < 0) {
+			ERROR("Failed to run \"btrfs_snapshot_wrapper\"");
+			return false;
+		}
+
+		TRACE("Created btrfs snapshot \"%s\" from \"%s\"", new->dest,
+		      orig->dest);
+		return true;
+	}
+
+	ret = btrfs_snapshot(orig->dest, new->dest);
+	if (ret < 0) {
+		SYSERROR("Failed to create btrfs snapshot \"%s\" from \"%s\"",
+			 new->dest, orig->dest);
+		return false;
+	}
+
+	TRACE("Created btrfs snapshot \"%s\" from \"%s\"", new->dest, orig->dest);
+	return true;
 }
 
 static int btrfs_do_destroy_subvol(const char *path)
 {
 	int ret, fd = -1;
-	size_t retlen;
 	struct btrfs_ioctl_vol_args  args;
 	char *p, *newfull = strdup(path);
 
@@ -473,13 +521,8 @@ static int btrfs_do_destroy_subvol(const char *path)
 	}
 
 	memset(&args, 0, sizeof(args));
-	retlen = strlcpy(args.name, p+1, BTRFS_SUBVOL_NAME_MAX);
-	if (retlen >= BTRFS_SUBVOL_NAME_MAX) {
-		free(newfull);
-		close(fd);
-		return -E2BIG;
-	}
-
+	strncpy(args.name, p+1, BTRFS_SUBVOL_NAME_MAX);
+	args.name[BTRFS_SUBVOL_NAME_MAX-1] = 0;
 	ret = ioctl(fd, BTRFS_IOC_SNAP_DESTROY, &args);
 	INFO("btrfs: snapshot destroy ioctl returned %d for %s", ret, path);
 	if (ret < 0 && errno == EPERM)
@@ -533,29 +576,22 @@ static bool update_tree_node(struct mytree_node *n, u64 id, u64 parent,
 {
 	if (id)
 		n->objid = id;
-
 	if (parent)
 		n->parentid = parent;
-
 	if (name) {
 		n->name = malloc(name_len + 1);
 		if (!n->name)
 			return false;
-
-		(void)strlcpy(n->name, name, name_len + 1);
+		strncpy(n->name, name, name_len);
+		n->name[name_len] = '\0';
 	}
-
 	if (dirname) {
-		size_t len;
-
-		len = strlen(dirname);
-		n->dirname = malloc(len + 1);
+		n->dirname = malloc(strlen(dirname) + 1);
 		if (!n->dirname) {
 			free(n->name);
 			return false;
 		}
-
-		(void)strlcpy(n->dirname, dirname, len + 1);
+		strcpy(n->dirname, dirname);
 	}
 	return true;
 }
@@ -602,7 +638,7 @@ static void free_btrfs_tree(struct my_btrfs_tree *tree)
 static bool do_remove_btrfs_children(struct my_btrfs_tree *tree, u64 root_id,
 				     const char *path)
 {
-	int i, ret;
+	int i;
 	char *newpath;
 	size_t len;
 
@@ -618,11 +654,7 @@ static bool do_remove_btrfs_children(struct my_btrfs_tree *tree, u64 root_id,
 				ERROR("Out of memory");
 				return false;
 			}
-			ret = snprintf(newpath, len, "%s/%s", path, tree->nodes[i].dirname);
-			if (ret < 0 || ret >= len) {
-				free(newpath);
-				return false;
-			}
+			snprintf(newpath, len, "%s/%s", path, tree->nodes[i].dirname);
 			if (!do_remove_btrfs_children(tree, tree->nodes[i].objid, newpath)) {
 				ERROR("Failed to prune %s\n", tree->nodes[i].name);
 				free(newpath);
@@ -783,21 +815,50 @@ bool btrfs_try_remove_subvol(const char *path)
 {
 	if (!btrfs_detect(path))
 		return false;
+
 	return btrfs_recursive_destroy(path) == 0;
 }
 
 int btrfs_destroy(struct lxc_storage *orig)
 {
-	return btrfs_recursive_destroy(orig->src);
+	char *src;
+
+	src = lxc_storage_get_path(orig->src, "btrfs");
+
+	return btrfs_recursive_destroy(src);
 }
 
 int btrfs_create(struct lxc_storage *bdev, const char *dest, const char *n,
 		 struct bdev_specs *specs)
 {
-	bdev->src = strdup(dest);
-	bdev->dest = strdup(dest);
-	if (!bdev->src || !bdev->dest)
-		return -1;
-	return btrfs_subvolume_create(bdev->dest);
-}
+	int ret;
+	size_t len;
 
+	len = strlen(dest) + 1;
+	/* strlen("btrfs:") */
+	len += 6;
+	bdev->src = malloc(len);
+	if (!bdev->src) {
+		ERROR("Failed to allocate memory");
+		return -1;
+	}
+
+	ret = snprintf(bdev->src, len, "btrfs:%s", dest);
+	if (ret < 0 || (size_t)ret >= len) {
+		ERROR("Failed to create string");
+		return -1;
+	}
+
+	bdev->dest = strdup(dest);
+	if (!bdev->dest) {
+		ERROR("Failed to duplicate string \"%s\"", dest);
+		return -1;
+	}
+
+	ret = btrfs_subvolume_create(bdev->dest);
+	if (ret < 0) {
+		SYSERROR("Failed to create btrfs subvolume \"%s\"", bdev->dest);
+	}
+
+	return ret;
+}
