@@ -39,95 +39,149 @@ int dir_clonepaths(struct lxc_storage *orig, struct lxc_storage *new,
 		   const char *lxcpath, int snap, uint64_t newsize,
 		   struct lxc_conf *conf)
 {
-	int len, ret;
+	char *src_no_prefix;
+	int ret;
+	size_t len;
 
 	if (snap) {
-		ERROR("directories cannot be snapshotted.  Try aufs or overlayfs.");
+		ERROR("Directories cannot be snapshotted");
 		return -1;
 	}
 
 	if (!orig->dest || !orig->src)
 		return -1;
 
-	len = strlen(lxcpath) + strlen(cname) + strlen("rootfs") + 3;
+	len = strlen(lxcpath) + strlen(cname) + strlen("rootfs") + 4 + 3;
 	new->src = malloc(len);
-	if (!new->src)
-		return -1;
-	ret = snprintf(new->src, len, "%s/%s/rootfs", lxcpath, cname);
-	if (ret < 0 || ret >= len)
-		return -1;
-	if ((new->dest = strdup(new->src)) == NULL)
+	if (!new->src) {
+		ERROR("Failed to allocate memory");
 		return -1;
+	}
 
+	ret = snprintf(new->src, len, "dir:%s/%s/rootfs", lxcpath, cname);
+	if (ret < 0 || (size_t)ret >= len) {
+		ERROR("Failed to create string");
+		return -1;
+	}
+
+	src_no_prefix = lxc_storage_get_path(new->src, new->type);
+	new->dest = strdup(src_no_prefix);
+	if (!new->dest) {
+		ERROR("Failed to duplicate string \"%s\"", new->src);
+		return -1;
+	}
+
+	TRACE("Created new path \"%s\" for dir storage driver", new->dest);
 	return 0;
 }
 
 int dir_create(struct lxc_storage *bdev, const char *dest, const char *n,
 	       struct bdev_specs *specs)
 {
+	int ret;
+	const char *src;
+	size_t len;
+
+	/* strlen("dir:") */
+	len = 4;
 	if (specs && specs->dir)
-		bdev->src = strdup(specs->dir);
+		src = specs->dir;
 	else
-		bdev->src = strdup(dest);
-	bdev->dest = strdup(dest);
-	if (!bdev->src || !bdev->dest) {
-		ERROR("Out of memory");
+		src = dest;
+
+	len += strlen(src) + 1;
+	bdev->src = malloc(len);
+	if (!bdev->src) {
+		ERROR("Failed to allocate memory");
 		return -1;
 	}
 
-	if (mkdir_p(bdev->src, 0755) < 0) {
-		ERROR("Error creating %s", bdev->src);
+	ret = snprintf(bdev->src, len, "dir:%s", src);
+	if (ret < 0 || (size_t)ret >= len) {
+		ERROR("Failed to create string");
 		return -1;
 	}
-	if (mkdir_p(bdev->dest, 0755) < 0) {
-		ERROR("Error creating %s", bdev->dest);
+
+	bdev->dest = strdup(dest);
+	if (!bdev->dest) {
+		ERROR("Failed to duplicate string \"%s\"", dest);
 		return -1;
 	}
 
+	ret = mkdir_p(dest, 0755);
+	if (ret < 0) {
+		ERROR("Failed to create directory \"%s\"", dest);
+		return -1;
+	}
+	TRACE("Created directory \"%s\"", dest);
+
 	return 0;
 }
 
 int dir_destroy(struct lxc_storage *orig)
 {
-	if (lxc_rmdir_onedev(orig->src, NULL) < 0)
+	int ret;
+	char *src;
+
+	src = lxc_storage_get_path(orig->src, orig->src);
+
+	ret = lxc_rmdir_onedev(src, NULL);
+	if (ret < 0) {
+		ERROR("Failed to delete \"%s\"", src);
 		return -1;
+	}
+
 	return 0;
 }
 
-int dir_detect(const char *path)
+bool dir_detect(const char *path)
 {
-	if (strncmp(path, "dir:", 4) == 0)
-		return 1; // take their word for it
+	if (!strncmp(path, "dir:", 4))
+		return true;
+
 	if (is_dir(path))
-		return 1;
-	return 0;
+		return true;
+
+	return false;
 }
 
 int dir_mount(struct lxc_storage *bdev)
 {
-	unsigned long mntflags;
-	char *mntdata;
 	int ret;
-	unsigned long mflags;
+	unsigned long mflags, mntflags;
+	char *src, *mntdata;
 
 	if (strcmp(bdev->type, "dir"))
 		return -22;
+
 	if (!bdev->src || !bdev->dest)
 		return -22;
 
-	if (parse_mntopts(bdev->mntopts, &mntflags, &mntdata) < 0) {
+	ret = parse_mntopts(bdev->mntopts, &mntflags, &mntdata);
+	if (ret < 0) {
+		ERROR("Failed to parse mount options \"%s\"", bdev->mntopts);
 		free(mntdata);
 		return -22;
 	}
 
-	ret = mount(bdev->src, bdev->dest, "bind", MS_BIND | MS_REC | mntflags, mntdata);
+	src = lxc_storage_get_path(bdev->src, bdev->type);
+
+	ret = mount(src, bdev->dest, "bind", MS_BIND | MS_REC | mntflags,
+		    mntdata);
 	if ((0 == ret) && (mntflags & MS_RDONLY)) {
-		DEBUG("remounting %s on %s with readonly options",
-			bdev->src ? bdev->src : "(none)", bdev->dest ? bdev->dest : "(none)");
-		mflags = add_required_remount_flags(bdev->src, bdev->dest, MS_BIND | MS_REC | mntflags | MS_REMOUNT);
-		ret = mount(bdev->src, bdev->dest, "bind", mflags, mntdata);
+		DEBUG("Remounting \"%s\" on \"%s\" readonly",
+		      src ? src : "(none)", bdev->dest ? bdev->dest : "(none)");
+		mflags = add_required_remount_flags(src, bdev->dest, MS_BIND | MS_REC | mntflags | MS_REMOUNT);
+		ret = mount(src, bdev->dest, "bind", mflags, mntdata);
 	}
 
+	if (ret < 0) {
+		SYSERROR("Failed to mount \"%s\" on \"%s\"", src, bdev->dest);
+		free(mntdata);
+		return -1;
+	}
+
+	TRACE("Mounted \"%s\" on \"%s\"", src, bdev->dest);
 	free(mntdata);
 	return ret;
 }
@@ -136,7 +190,9 @@ int dir_umount(struct lxc_storage *bdev)
 {
 	if (strcmp(bdev->type, "dir"))
 		return -22;
+
 	if (!bdev->src || !bdev->dest)
 		return -22;
+
 	return umount(bdev->dest);
 }
