@@ -61,10 +61,6 @@
 #include "utils.h"
 #include "zfs.h"
 
-#ifndef HAVE_STRLCPY
-#include "include/strlcpy.h"
-#endif
-
 #ifndef BLKGETSIZE64
 #define BLKGETSIZE64 _IOR(0x12, 114, size_t)
 #endif
@@ -79,6 +75,8 @@ static const struct lxc_storage_ops aufs_ops = {
     .clone_paths = &aufs_clonepaths,
     .destroy = &aufs_destroy,
     .create = &aufs_create,
+    .copy = NULL,
+    .snapshot = NULL,
     .can_snapshot = true,
     .can_backup = true,
 };
@@ -91,6 +89,8 @@ static const struct lxc_storage_ops btrfs_ops = {
     .clone_paths = &btrfs_clonepaths,
     .destroy = &btrfs_destroy,
     .create = &btrfs_create,
+    .copy = &btrfs_create_clone,
+    .snapshot = &btrfs_create_snapshot,
     .can_snapshot = true,
     .can_backup = true,
 };
@@ -103,6 +103,8 @@ static const struct lxc_storage_ops dir_ops = {
     .clone_paths = &dir_clonepaths,
     .destroy = &dir_destroy,
     .create = &dir_create,
+    .copy = NULL,
+    .snapshot = NULL,
     .can_snapshot = false,
     .can_backup = true,
 };
@@ -115,6 +117,8 @@ static const struct lxc_storage_ops loop_ops = {
     .clone_paths = &loop_clonepaths,
     .destroy = &loop_destroy,
     .create = &loop_create,
+    .copy = NULL,
+    .snapshot = NULL,
     .can_snapshot = false,
     .can_backup = true,
 };
@@ -127,6 +131,8 @@ static const struct lxc_storage_ops lvm_ops = {
     .clone_paths = &lvm_clonepaths,
     .destroy = &lvm_destroy,
     .create = &lvm_create,
+    .copy = &lvm_create_clone,
+    .snapshot = &lvm_create_snapshot,
     .can_snapshot = true,
     .can_backup = false,
 };
@@ -139,6 +145,8 @@ const struct lxc_storage_ops nbd_ops = {
     .clone_paths = &nbd_clonepaths,
     .destroy = &nbd_destroy,
     .create = &nbd_create,
+    .copy = NULL,
+    .snapshot = NULL,
     .can_snapshot = true,
     .can_backup = false,
 };
@@ -151,6 +159,8 @@ static const struct lxc_storage_ops ovl_ops = {
     .clone_paths = &ovl_clonepaths,
     .destroy = &ovl_destroy,
     .create = &ovl_create,
+    .copy = NULL,
+    .snapshot = NULL,
     .can_snapshot = true,
     .can_backup = true,
 };
@@ -163,6 +173,8 @@ static const struct lxc_storage_ops rbd_ops = {
     .clone_paths = &rbd_clonepaths,
     .destroy = &rbd_destroy,
     .create = &rbd_create,
+    .copy = NULL,
+    .snapshot = NULL,
     .can_snapshot = false,
     .can_backup = false,
 };
@@ -175,6 +187,8 @@ static const struct lxc_storage_ops zfs_ops = {
     .clone_paths = &zfs_clonepaths,
     .destroy = &zfs_destroy,
     .create = &zfs_create,
+    .copy = &zfs_copy,
+    .snapshot = &zfs_snapshot,
     .can_snapshot = true,
     .can_backup = true,
 };
@@ -185,12 +199,13 @@ struct lxc_storage_type {
 };
 
 static const struct lxc_storage_type bdevs[] = {
+	{ .name = "dir",       .ops = &dir_ops,   },
 	{ .name = "zfs",       .ops = &zfs_ops,   },
 	{ .name = "lvm",       .ops = &lvm_ops,   },
 	{ .name = "rbd",       .ops = &rbd_ops,   },
 	{ .name = "btrfs",     .ops = &btrfs_ops, },
-	{ .name = "dir",       .ops = &dir_ops,   },
 	{ .name = "aufs",      .ops = &aufs_ops,  },
+	{ .name = "overlay",   .ops = &ovl_ops,   },
 	{ .name = "overlayfs", .ops = &ovl_ops,   },
 	{ .name = "loop",      .ops = &loop_ops,  },
 	{ .name = "nbd",       .ops = &nbd_ops,   },
@@ -259,6 +274,11 @@ struct lxc_storage *storage_get(const char *type)
 	bdev->ops = bdevs[i].ops;
 	bdev->type = bdevs[i].name;
 
+	if (!strcmp(bdev->type, "aufs"))
+		WARN("The \"aufs\" driver will is deprecated and will soon be "
+		     "removed. For similar functionality see the \"overlay\" "
+		     "storage driver");
+
 	return bdev;
 }
 
@@ -300,40 +320,35 @@ bool storage_can_backup(struct lxc_conf *conf)
 /* If we're not snaphotting, then storage_copy becomes a simple case of mount
  * the original, mount the new, and rsync the contents.
  */
-struct lxc_storage *storage_copy(struct lxc_container *c0, const char *cname,
+struct lxc_storage *storage_copy(struct lxc_container *c, const char *cname,
 				 const char *lxcpath, const char *bdevtype,
 				 int flags, const char *bdevdata,
-				 uint64_t newsize, int *needs_rdep)
+				 uint64_t newsize, bool *needs_rdep)
 {
-	struct lxc_storage *orig, *new;
-	pid_t pid;
 	int ret;
+	struct lxc_storage *orig, *new;
+	char *src_no_prefix;
 	bool snap = flags & LXC_CLONE_SNAPSHOT;
 	bool maybe_snap = flags & LXC_CLONE_MAYBE_SNAPSHOT;
 	bool keepbdevtype = flags & LXC_CLONE_KEEPBDEVTYPE;
-	const char *src = c0->lxc_conf->rootfs.path;
-	const char *oldname = c0->name;
-	const char *oldpath = c0->config_path;
-	struct rsync_data data;
+	const char *src = c->lxc_conf->rootfs.path;
+	const char *oldname = c->name;
+	const char *oldpath = c->config_path;
+	struct rsync_data data = {0};
+	char cmd_output[MAXPATHLEN] = {0};
 
 	/* If the container name doesn't show up in the rootfs path, then we
 	 * don't know how to come up with a new name.
 	 */
-	if (!src) {
-		ERROR("No rootfs specified");
+	if (!strstr(src, oldname)) {
+		ERROR("Original rootfs path \"%s\" does not include container "
+		      "name \"%s\"", src, oldname);
 		return NULL;
 	}
 
-	if (strstr(src, oldname) == NULL) {
-		ERROR(
-		    "original rootfs path %s doesn't include container name %s",
-		    src, oldname);
-		return NULL;
-	}
-
-	orig = storage_init(c0->lxc_conf, src, NULL, NULL);
+	orig = storage_init(c->lxc_conf, src, NULL, NULL);
 	if (!orig) {
-		ERROR("failed to detect blockdev type for %s", src);
+		ERROR("Failed to detect storage driver for \"%s\"", src);
 		return NULL;
 	}
 
@@ -346,133 +361,171 @@ struct lxc_storage *storage_copy(struct lxc_container *c0, const char *cname,
 		orig->dest = malloc(len);
 		if (!orig->dest) {
 			ERROR("Failed to allocate memory");
-			storage_put(orig);
-			return NULL;
+			goto on_error_put_orig;
 		}
 
 		ret = snprintf(orig->dest, len, "%s/%s/rootfs", oldpath, oldname);
 		if (ret < 0 || (size_t)ret >= len) {
 			ERROR("Failed to create string");
-			storage_put(orig);
-			return NULL;
+			goto on_error_put_orig;
 		}
-		ret = stat(orig->dest, &sb);
 
-		if (ret < 0 && errno == ENOENT)
-			if (mkdir_p(orig->dest, 0755) < 0)
-				WARN("Error creating '%s', continuing.",
-				     orig->dest);
+		ret = stat(orig->dest, &sb);
+		if (ret < 0 && errno == ENOENT) {
+			ret = mkdir_p(orig->dest, 0755);
+			if (ret < 0)
+				WARN("Failed to create directoy \"%s\"", orig->dest);
+		}
 	}
 
-	/*
-	 * special case for snapshot - if caller requested maybe_snapshot and
-	 * keepbdevtype and backing store is directory, then proceed with a copy
-	 * clone rather than returning error
+	/* Special case for snapshot. If the caller requested maybe_snapshot and
+	 * keepbdevtype and the backing store is directory, then proceed with a
+	 * a copy clone rather than returning error.
 	 */
 	if (maybe_snap && keepbdevtype && !bdevtype && !orig->ops->can_snapshot)
 		snap = false;
 
-	/* If newtype is NULL and snapshot is set, then use overlayfs. */
-	if (!bdevtype && !keepbdevtype && snap && (!strcmp(orig->type, "dir") || !strcmp(orig->type, "overlayfs")))
-		bdevtype = "overlayfs";
+	/* If newtype is NULL and snapshot is set, then use overlay. */
+	if (!bdevtype && !keepbdevtype && snap && !strcmp(orig->type, "dir"))
+		bdevtype = "overlay";
 
-	if (am_guest_unpriv() && !unpriv_snap_allowed(orig, bdevtype, snap, maybe_snap)) {
+	if (am_unpriv() && !unpriv_snap_allowed(orig, bdevtype, snap, maybe_snap)) {
 		ERROR("Unsupported snapshot type \"%s\" for unprivileged users",
 		      bdevtype ? bdevtype : "(null)");
-		storage_put(orig);
-		return NULL;
+		goto on_error_put_orig;
 	}
 
-	*needs_rdep = 0;
-	if (bdevtype && strcmp(orig->type, "dir") == 0 &&
-	    (strcmp(bdevtype, "aufs") == 0 ||
-	     strcmp(bdevtype, "overlayfs") == 0)) {
-		*needs_rdep = 1;
-	} else if (snap && strcmp(orig->type, "lvm") == 0 &&
-		   !lvm_is_thin_volume(orig->src)) {
-		*needs_rdep = 1;
-	}
+	*needs_rdep = false;
+	if (bdevtype) {
+		if (snap && !strcmp(orig->type, "lvm") &&
+		    !lvm_is_thin_volume(orig->src))
+			*needs_rdep = true;
+		else if (!strcmp(bdevtype, "overlay") ||
+			 !strcmp(bdevtype, "overlayfs"))
+			*needs_rdep = true;
+	} else {
+		if (!snap && strcmp(oldpath, lxcpath))
+			bdevtype = "dir";
+		else
+			bdevtype = orig->type;
 
-	if (strcmp(oldpath, lxcpath) && !bdevtype && strcmp(orig->type, "overlayfs"))
-		bdevtype = "dir";
-	else if (!bdevtype)
-		bdevtype = orig->type;
+		if (!strcmp(bdevtype, "overlay") ||
+		    !strcmp(bdevtype, "overlayfs"))
+			*needs_rdep = true;
+	}
 
 	/* get new bdev type */
 	new = storage_get(bdevtype);
 	if (!new) {
-		ERROR("no such block device type: %s",
+		ERROR("Failed to initialize \"%s\" storage driver",
 		      bdevtype ? bdevtype : orig->type);
-		storage_put(orig);
-		return NULL;
+		goto on_error_put_orig;
+	}
+	TRACE("Initialized \"%s\" storage driver", new->type);
+
+	/* create new paths */
+	ret = new->ops->clone_paths(orig, new, oldname, cname, oldpath, lxcpath,
+				    snap, newsize, c->lxc_conf);
+	if (ret < 0) {
+		ERROR("Failed creating new paths for clone of \"%s\"", src);
+		goto on_error_put_new;
 	}
 
-	if (new->ops->clone_paths(orig, new, oldname, cname, oldpath, lxcpath,
-				  snap, newsize, c0->lxc_conf) < 0) {
-		ERROR("failed getting pathnames for cloned storage: %s", src);
-		goto err;
+	/* When we create an overlay snapshot of an overlay container in the
+	 * snapshot directory under "<lxcpath>/<name>/snaps/" we don't need to
+	 * record a dependency. If we would restore would also fail.
+	 */
+	if ((!strcmp(new->type, "overlay") ||
+	     !strcmp(new->type, "overlayfs")) &&
+	    ret == LXC_CLONE_SNAPSHOT)
+		*needs_rdep = false;
+
+	/* btrfs */
+	if (!strcmp(orig->type, "btrfs") && !strcmp(new->type, "btrfs")) {
+		bool bret = false;
+		if (snap || btrfs_same_fs(orig->dest, new->dest) == 0)
+			bret = new->ops->snapshot(c->lxc_conf, orig, new, 0);
+		else
+			bret = new->ops->copy(c->lxc_conf, orig, new, 0);
+		if (!bret)
+			goto on_error_put_new;
+
+		goto on_success;
 	}
 
-	if (am_guest_unpriv() && chown_mapped_root(new->src, c0->lxc_conf) < 0)
-		WARN("Failed to update ownership of %s", new->dest);
+	/* lvm */
+	if (!strcmp(orig->type, "lvm") && !strcmp(new->type, "lvm")) {
+		bool bret = false;
+		if (snap)
+			bret = new->ops->snapshot(c->lxc_conf, orig,
+							 new, newsize);
+		else
+			bret = new->ops->copy(c->lxc_conf, orig, new, newsize);
+		if (!bret)
+			goto on_error_put_new;
+
+		goto on_success;
+	}
+
+	/* zfs */
+	if (!strcmp(orig->type, "zfs") && !strcmp(new->type, "zfs")) {
+		bool bret = false;
+
+		if (snap)
+			bret = new->ops->snapshot(c->lxc_conf, orig, new,
+						  newsize);
+		else
+			bret = new->ops->copy(c->lxc_conf, orig, new, newsize);
+		if (!bret)
+			goto on_error_put_new;
+
+		goto on_success;
+	}
+
+	if (strcmp(bdevtype, "btrfs")) {
+		if (!strcmp(new->type, "overlay") || !strcmp(new->type, "overlayfs"))
+			src_no_prefix = ovl_get_lower(new->src);
+		else
+			src_no_prefix = lxc_storage_get_path(new->src, new->type);
+
+		if (am_unpriv()) {
+			ret = chown_mapped_root(src_no_prefix, c->lxc_conf);
+			if (ret < 0)
+				WARN("Failed to chown \"%s\"", new->src);
+		}
+	}
 
 	if (snap)
-		return new;
-
-	/*
-	 * https://github.com/lxc/lxc/issues/131
-	 * Use btrfs snapshot feature instead of rsync to restore if both orig
-	 * and new are btrfs
-	 */
-	if (bdevtype && strcmp(orig->type, "btrfs") == 0 &&
-	    strcmp(new->type, "btrfs") == 0 &&
-	    btrfs_same_fs(orig->dest, new->dest) == 0) {
-		if (btrfs_destroy(new) < 0) {
-			ERROR("Error destroying %s subvolume", new->dest);
-			goto err;
-		}
-		if (mkdir_p(new->dest, 0755) < 0) {
-			ERROR("Error creating %s directory", new->dest);
-			goto err;
-		}
-		if (btrfs_snapshot(orig->dest, new->dest) < 0) {
-			ERROR("Error restoring %s to %s", orig->dest,
-			      new->dest);
-			goto err;
-		}
-		storage_put(orig);
-		return new;
-	}
-
-	pid = fork();
-	if (pid < 0) {
-		SYSERROR("fork");
-		goto err;
-	}
-
-	if (pid > 0) {
-		int ret = wait_for_pid(pid);
-		storage_put(orig);
-		if (ret < 0) {
-			storage_put(new);
-			return NULL;
-		}
-		return new;
-	}
+		goto on_success;
 
+	/* rsync the contents from source to target */
 	data.orig = orig;
 	data.new = new;
-	if (am_guest_unpriv())
-		ret = userns_exec_full(c0->lxc_conf, rsync_rootfs_wrapper,
-				       &data, "rsync_rootfs_wrapper");
+	if (am_unpriv())
+		ret = userns_exec_1(c->lxc_conf, lxc_storage_rsync_exec_wrapper,
+				    &data, "lxc_storage_rsync_exec_wrapper");
 	else
-		ret = rsync_rootfs(&data);
+		ret = run_command(cmd_output, sizeof(cmd_output),
+				  lxc_storage_rsync_exec_wrapper, (void *)&data);
+	if (ret < 0) {
+		ERROR("Failed to rsync from \"%s\" into \"%s\"%s%s", orig->dest,
+		      new->dest,
+		      cmd_output[0] != '\0' ? ": " : "",
+		      cmd_output[0] != '\0' ? cmd_output : "");
+		goto on_error_put_new;
+	}
 
-	exit(ret == 0 ? 0 : 1);
-
-err:
+on_success:
 	storage_put(orig);
+
+	return new;
+
+on_error_put_new:
 	storage_put(new);
+
+on_error_put_orig:
+	storage_put(orig);
+
 	return NULL;
 }
 
@@ -488,21 +541,20 @@ err:
 struct lxc_storage *storage_create(const char *dest, const char *type,
 				   const char *cname, struct bdev_specs *specs)
 {
-	int ret;
 	struct lxc_storage *bdev;
 	char *best_options[] = {"btrfs", "zfs", "lvm", "dir", "rbd", NULL};
 
 	if (!type)
 		return do_storage_create(dest, "dir", cname, specs);
 
-	ret = strcmp(type, "best");
-	if (ret == 0) {
+	if (strcmp(type, "best") == 0) {
 		int i;
 		/* Try for the best backing store type, according to our
 		 * opinionated preferences.
 		 */
 		for (i = 0; best_options[i]; i++) {
-			bdev = do_storage_create(dest, best_options[i], cname, specs);
+			bdev = do_storage_create(dest, best_options[i], cname,
+						 specs);
 			if (bdev)
 				return bdev;
 		}
@@ -511,19 +563,12 @@ struct lxc_storage *storage_create(const char *dest, const char *type,
 	}
 
 	/* -B lvm,dir */
-	if (strchr(type, ',')) {
-		char *dup, *token;
-		char *saveptr = NULL;
-		size_t len;
-
-		len = strlen(type);
-		dup = alloca(len + 1);
-		(void)strlcpy(dup, type, len + 1);
-
+	if (strchr(type, ',') != NULL) {
+		char *dup = alloca(strlen(type) + 1), *saveptr = NULL, *token;
+		strcpy(dup, type);
 		for (token = strtok_r(dup, ",", &saveptr); token;
 		     token = strtok_r(NULL, ",", &saveptr)) {
-			bdev = do_storage_create(dest, token, cname, specs);
-			if (bdev)
+			if ((bdev = do_storage_create(dest, token, cname, specs)))
 				return bdev;
 		}
 	}
@@ -553,6 +598,8 @@ struct lxc_storage *storage_init(struct lxc_conf *conf, const char *src,
 	struct lxc_storage *bdev;
 	const struct lxc_storage_type *q;
 
+	BUILD_BUG_ON(LXC_STORAGE_INTERNAL_OVERLAY_RESTORE <= LXC_CLONE_MAXFLAGS);
+
 	if (!src)
 		src = conf->rootfs.path;
 
@@ -579,6 +626,11 @@ struct lxc_storage *storage_init(struct lxc_conf *conf, const char *src,
 	if (strcmp(bdev->type, "nbd") == 0)
 		bdev->nbd_idx = conf->nbd_idx;
 
+	if (!strcmp(bdev->type, "aufs"))
+		WARN("The \"aufs\" driver will is deprecated and will soon be "
+		     "removed. For similar functionality see the \"overlay\" "
+		     "storage driver");
+
 	return bdev;
 }
 
@@ -626,8 +678,21 @@ bool rootfs_is_blockdev(struct lxc_conf *conf)
 
 	if (strcmp(q->name, "lvm") == 0 ||
 	    strcmp(q->name, "loop") == 0 ||
-	    strcmp(q->name, "nbd") == 0)
+	    strcmp(q->name, "nbd") == 0 ||
+	    strcmp(q->name, "rbd") == 0 ||
+	    strcmp(q->name, "zfs") == 0)
 		return true;
 
 	return false;
 }
+
+char *lxc_storage_get_path(char *src, const char *prefix)
+{
+	size_t prefix_len;
+
+	prefix_len = strlen(prefix);
+	if (!strncmp(src, prefix, prefix_len) && (*(src + prefix_len) == ':'))
+		return (src + prefix_len + 1);
+
+	return src;
+}
