@@ -61,14 +61,17 @@
 #endif
 
 #define DEFAULT_FS_SIZE 1073741824
-#define DEFAULT_FSTYPE "ext4"
+#define DEFAULT_FSTYPE "ext3"
+
+#define LXC_STORAGE_INTERNAL_OVERLAY_RESTORE  (1 << 6)
 
 struct lxc_storage;
 
 struct lxc_storage_ops {
 	/* detect whether path is of this bdev type */
-	int (*detect)(const char *path);
-	// mount requires src and dest to be set.
+	bool (*detect)(const char *path);
+
+	/* mount requires src and dest to be set. */
 	int (*mount)(struct lxc_storage *bdev);
 	int (*umount)(struct lxc_storage *bdev);
 	int (*destroy)(struct lxc_storage *bdev);
@@ -79,6 +82,10 @@ struct lxc_storage_ops {
 			   const char *oldname, const char *cname,
 			   const char *oldpath, const char *lxcpath, int snap,
 			   uint64_t newsize, struct lxc_conf *conf);
+	bool (*copy)(struct lxc_conf *conf, struct lxc_storage *orig,
+		     struct lxc_storage *new, uint64_t newsize);
+	bool (*snapshot)(struct lxc_conf *conf, struct lxc_storage *orig,
+			 struct lxc_storage *new, uint64_t newsize);
 	bool can_snapshot;
 	bool can_backup;
 };
@@ -99,6 +106,7 @@ struct lxc_storage {
 	int lofd;
 	/* index for the connected nbd device. */
 	int nbd_idx;
+	int flags;
 };
 
 extern bool storage_is_dir(struct lxc_conf *conf, const char *path);
@@ -121,12 +129,16 @@ extern struct lxc_storage *storage_copy(struct lxc_container *c0,
 					const char *cname, const char *lxcpath,
 					const char *bdevtype, int flags,
 					const char *bdevdata, uint64_t newsize,
-					int *needs_rdep);
+					bool *needs_rdep);
 extern struct lxc_storage *storage_create(const char *dest, const char *type,
 					  const char *cname,
 					  struct bdev_specs *specs);
 extern void storage_put(struct lxc_storage *bdev);
 extern bool storage_destroy(struct lxc_conf *conf);
-extern bool rootfs_is_blockdev(struct lxc_conf *conf);
 
-#endif // __LXC_STORAGE_H
+/* callback function to be used with userns_exec_1() */
+extern int storage_destroy_wrapper(void *data);
+extern bool rootfs_is_blockdev(struct lxc_conf *conf);
+extern char *lxc_storage_get_path(char *src, const char *prefix);
+
+#endif /* #define __LXC_STORAGE_H */
