@@ -21,29 +21,29 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  */
 #define _GNU_SOURCE
-#include <errno.h>
-#include <libgen.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <errno.h>
+#include <libgen.h>
 #include <string.h>
 #include <unistd.h>
-#include <sys/param.h>
-#include <sys/stat.h>
-#include <sys/types.h>
-#include <sys/wait.h>
-
 #include <lxc/lxccontainer.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <sys/param.h>
 
-#include "arguments.h"
 #include "caps.h"
-#include "conf.h"
-#include "config.h"
-#include "confile.h"
-#include "log.h"
 #include "lxc.h"
+#include "log.h"
+#include "conf.h"
+#include "confile.h"
+#include "arguments.h"
+#include "config.h"
 #include "start.h"
 #include "utils.h"
 
+lxc_log_define(lxc_execute_ui, lxc);
+
 static struct lxc_list defines;
 
 static int my_checker(const struct lxc_arguments* args)
@@ -131,37 +131,25 @@ int main(int argc, char *argv[])
 
 	c = lxc_container_new(my_args.name, my_args.lxcpath[0]);
 	if (!c) {
-		fprintf(stderr, "Failed to create lxc_container\n");
+		ERROR("Failed to create lxc_container");
 		exit(EXIT_FAILURE);
 	}
 
 	if (my_args.rcfile) {
 		c->clear_config(c);
 		if (!c->load_config(c, my_args.rcfile)) {
-			fprintf(stderr, "Failed to load rcfile\n");
+			ERROR("Failed to load rcfile");
 			lxc_container_put(c);
 			exit(EXIT_FAILURE);
 		}
 		c->configfile = strdup(my_args.rcfile);
 		if (!c->configfile) {
-			fprintf(stderr, "Out of memory setting new config filename\n");
+			ERROR("Out of memory setting new config filename");
 			lxc_container_put(c);
 			exit(EXIT_FAILURE);
 		}
 	}
 
-	if (!c->lxc_conf) {
-		fprintf(stderr, "Executing a container with no configuration file may crash the host\n");
-		lxc_container_put(c);
-		exit(EXIT_FAILURE);
-	}
-
-	ret = lxc_config_define_load(&defines, c->lxc_conf);
-	if (ret) {
-		lxc_container_put(c);
-		exit(EXIT_FAILURE);
-	}
-
 	if (my_args.uid)
 		c->lxc_conf->init_uid = my_args.uid;
 
@@ -170,20 +158,11 @@ int main(int argc, char *argv[])
 
 	c->daemonize = false;
 	bret = c->start(c, 1, my_args.argv);
+	ret = c->error_num;
 	lxc_container_put(c);
 	if (!bret) {
-		fprintf(stderr, "Failed run an application inside container\n");
+		ERROR("Failed run an application inside container");
 		exit(EXIT_FAILURE);
 	}
-	if (c->daemonize)
-		exit(EXIT_SUCCESS);
-	else {
-		if (WIFEXITED(c->error_num)) {
-			exit(WEXITSTATUS(c->error_num));
-		} else {
-			/* Try to die with the same signal the task did. */
-			kill(0, WTERMSIG(c->error_num));
-			exit(EXIT_FAILURE);
-		}
-	}
+	exit(ret);
 }
