@@ -37,6 +37,8 @@
 #include "commands.h"
 #include "arguments.h"
 
+lxc_log_define(lxc_info_ui, lxc);
+
 static bool ips;
 static bool state;
 static bool pid;
@@ -153,15 +155,15 @@ static void print_net_stats(struct lxc_container *c)
 	char buf[256];
 
 	for(netnr = 0; ;netnr++) {
-		sprintf(buf, "lxc.network.%d.type", netnr);
+		sprintf(buf, "lxc.net.%d.type", netnr);
 		type = c->get_running_config_item(c, buf);
 		if (!type)
 			break;
 
 		if (!strcmp(type, "veth")) {
-			sprintf(buf, "lxc.network.%d.veth.pair", netnr);
+			sprintf(buf, "lxc.net.%d.veth.pair", netnr);
 		} else {
-			sprintf(buf, "lxc.network.%d.link", netnr);
+			sprintf(buf, "lxc.net.%d.link", netnr);
 		}
 		free(type);
 		ifname = c->get_running_config_item(c, buf);
@@ -176,7 +178,6 @@ static void print_net_stats(struct lxc_container *c)
 		snprintf(path, sizeof(path), "/sys/class/net/%s/statistics/rx_bytes", ifname);
 		rc = lxc_read_from_file(path, buf, sizeof(buf));
 		if (rc > 0) {
-			buf[rc - 1] = '\0';
 			str_chomp(buf);
 			rx_bytes = str_size_humanize(buf, sizeof(buf));
 			printf("%-15s %s\n", " TX bytes:", buf);
@@ -186,7 +187,6 @@ static void print_net_stats(struct lxc_container *c)
 		snprintf(path, sizeof(path), "/sys/class/net/%s/statistics/tx_bytes", ifname);
 		rc = lxc_read_from_file(path, buf, sizeof(buf));
 		if (rc > 0) {
-			buf[rc - 1] = '\0';
 			str_chomp(buf);
 			tx_bytes = str_size_humanize(buf, sizeof(buf));
 			printf("%-15s %s\n", " RX bytes:", buf);
