@@ -546,7 +546,12 @@ static int ls_get(struct ls **m, size_t *size, const struct lxc_arguments *args,
 			 * need a path-extractor function. We face the same
 			 * problem with the ovl_mkdir() function in
 			 * lxcoverlay.{c,h}. */
-			char *curr_path = ls_get_config_item(c, "lxc.rootfs", running);
+			char *curr_path = ls_get_config_item(c, "lxc.rootfs.path", running);
+			/* REMOVE IN LXC 3.0
+			   legacy rootfs key
+			   */
+			if (!curr_path)
+				curr_path = ls_get_config_item(c, "lxc.rootfs", running);
 			if (!curr_path)
 				goto put_and_next;
 
@@ -672,18 +677,22 @@ static char *ls_get_interface(struct lxc_container *c)
  */
 static double ls_get_swap(struct lxc_container *c)
 {
+	char *stat, *swap, *tmp;
 	unsigned long long int num = 0;
-	char *stat = ls_get_cgroup_item(c, "memory.stat");
+
+	stat = ls_get_cgroup_item(c, "memory.stat");
 	if (!stat)
 		goto out;
 
-	char *swap = strstr(stat, "\nswap");
+	swap = strstr(stat, "\nswap");
 	if (!swap)
 		goto out;
 
-	swap = 1 + swap + 4 + 1; // start_of_swap_value = '\n' + strlen(swap) + ' '
+	/* start_of_swap_value = '\n' + strlen(swap) + ' ' */
+	swap = 1 + swap + 4 + 1;
 
-	char *tmp = strchr(swap, '\n'); // find end of swap value
+	/* find end of swap value */
+	tmp = strchr(swap, '\n');
 	if (!tmp)
 		goto out;
 
