@@ -50,6 +50,12 @@
 #include "confile.h"
 #include "arguments.h"
 
+#define OPT_SHARE_NET OPT_USAGE + 1
+#define OPT_SHARE_IPC OPT_USAGE + 2
+#define OPT_SHARE_UTS OPT_USAGE + 3
+
+lxc_log_define(lxc_start_ui, lxc);
+
 static struct lxc_list defines;
 
 static int ensure_path(char **confpath, const char *path)
@@ -61,7 +67,7 @@ static int ensure_path(char **confpath, const char *path)
 		if (access(path, W_OK)) {
 			fd = creat(path, 0600);
 			if (fd < 0 && errno != EEXIST) {
-				fprintf(stderr, "failed to create '%s'\n", path);
+				SYSERROR("failed to create '%s'", path);
 				goto err;
 			}
 			if (fd >= 0)
@@ -70,7 +76,7 @@ static int ensure_path(char **confpath, const char *path)
 
 		fullpath = realpath(path, NULL);
 		if (!fullpath) {
-			fprintf(stderr, "failed to get the real path of '%s'\n", path);
+			SYSERROR("failed to get the real path of '%s'", path);
 			goto err;
 		}
 
@@ -89,19 +95,19 @@ static int pid_from_lxcname(const char *lxcname_or_pid, const char *lxcpath) {
 		struct lxc_container *s;
 		s = lxc_container_new(lxcname_or_pid, lxcpath);
 		if (!s) {
-			fprintf(stderr, "'%s' is not a valid pid nor a container name\n", lxcname_or_pid);
+			SYSERROR("'%s' is not a valid pid nor a container name", lxcname_or_pid);
 			return -1;
 		}
 
 		if (!s->may_control(s)) {
-			fprintf(stderr, "Insufficient privileges to control container '%s'\n", s->name);
+			SYSERROR("Insufficient privileges to control container '%s'", s->name);
 			lxc_container_put(s);
 			return -1;
 		}
 
 		pid = s->init_pid(s);
 		if (pid < 1) {
-			fprintf(stderr, "Is container '%s' running?\n", s->name);
+			SYSERROR("Is container '%s' running?", s->name);
 			lxc_container_put(s);
 			return -1;
 		}
@@ -109,7 +115,7 @@ static int pid_from_lxcname(const char *lxcname_or_pid, const char *lxcpath) {
 		lxc_container_put(s);
 	}
 	if (kill(pid, 0) < 0) {
-		fprintf(stderr, "Can't send signal to pid %d\n", pid);
+		SYSERROR("Can't send signal to pid %d", pid);
 		return -1;
 	}
 
@@ -123,7 +129,7 @@ static int open_ns(int pid, const char *ns_proc_name) {
 
 	fd = open(path, O_RDONLY);
 	if (fd < 0) {
-		fprintf(stderr, "failed to open %s\n", path);
+		SYSERROR("failed to open %s", path);
 		return -1;
 	}
 	return fd;
@@ -192,7 +198,7 @@ Options :\n\
 
 int main(int argc, char *argv[])
 {
-	int i, err = EXIT_FAILURE;
+	int err = EXIT_FAILURE;
 	struct lxc_conf *conf;
 	struct lxc_log log;
 	char *const *args;
@@ -246,18 +252,18 @@ int main(int argc, char *argv[])
 		rcfile = (char *)my_args.rcfile;
 		c = lxc_container_new(my_args.name, lxcpath);
 		if (!c) {
-			fprintf(stderr, "Failed to create lxc_container\n");
+			ERROR("Failed to create lxc_container");
 			exit(err);
 		}
 		c->clear_config(c);
 		if (!c->load_config(c, rcfile)) {
-			fprintf(stderr, "Failed to load rcfile\n");
+			ERROR("Failed to load rcfile");
 			lxc_container_put(c);
 			exit(err);
 		}
 		c->configfile = strdup(my_args.rcfile);
 		if (!c->configfile) {
-			fprintf(stderr, "Out of memory setting new config filename\n");
+			ERROR("Out of memory setting new config filename");
 			goto out;
 		}
 	} else {
@@ -265,9 +271,10 @@ int main(int argc, char *argv[])
 
 		rc = asprintf(&rcfile, "%s/%s/config", lxcpath, my_args.name);
 		if (rc == -1) {
-			fprintf(stderr, "failed to allocate memory\n");
+			SYSERROR("failed to allocate memory");
 			exit(err);
 		}
+		INFO("using rcfile %s", rcfile);
 
 		/* container configuration does not exist */
 		if (access(rcfile, F_OK)) {
@@ -276,7 +283,7 @@ int main(int argc, char *argv[])
 		}
 		c = lxc_container_new(my_args.name, lxcpath);
 		if (!c) {
-			fprintf(stderr, "Failed to create lxc_container\n");
+			ERROR("Failed to create lxc_container");
 			exit(err);
 		}
 	}
@@ -293,7 +300,7 @@ int main(int argc, char *argv[])
 	}
 
 	if (c->is_running(c)) {
-		fprintf(stderr, "Container is already running.\n");
+		ERROR("Container is already running.");
 		err = EXIT_SUCCESS;
 		goto out;
 	}
@@ -309,25 +316,28 @@ int main(int argc, char *argv[])
 		goto out;
 
 	if (!rcfile && !strcmp("/sbin/init", args[0])) {
-		fprintf(stderr, "Executing '/sbin/init' with no configuration file may crash the host\n");
+		ERROR("Executing '/sbin/init' with no configuration file may crash the host");
+		goto out;
+	}
+
+	if (ensure_path(&conf->console.path, my_args.console) < 0) {
+		ERROR("failed to ensure console path '%s'", my_args.console);
+		goto out;
+	}
+
+	if (ensure_path(&conf->console.log_path, my_args.console_log) < 0) {
+		ERROR("failed to ensure console log '%s'", my_args.console_log);
 		goto out;
 	}
 
 	if (my_args.pidfile != NULL) {
 		if (ensure_path(&c->pidfile, my_args.pidfile) < 0) {
-			fprintf(stderr, "failed to ensure pidfile '%s'\n", my_args.pidfile);
+			ERROR("failed to ensure pidfile '%s'", my_args.pidfile);
 			goto out;
 		}
 	}
 
-	if (my_args.console)
-		if (!c->set_config_item(c, "lxc.console.path", my_args.console))
-			goto out;
-
-	if (my_args.console_log)
-		if (!c->set_config_item(c, "lxc.console.logfile", my_args.console_log))
-			goto out;
-
+	int i;
 	for (i = 0; i < LXC_NS_MAX; i++) {
 		if (my_args.share_ns[i] == NULL)
 			continue;
@@ -355,11 +365,11 @@ int main(int argc, char *argv[])
 		err = c->start(c, 0, args) ? EXIT_SUCCESS : EXIT_FAILURE;
 
 	if (err) {
-		fprintf(stderr, "The container failed to start.\n");
+		ERROR("The container failed to start.");
 		if (my_args.daemonize)
-			fprintf(stderr, "To get more details, run the container in foreground mode.\n");
-		fprintf(stderr, "Additional information can be obtained by setting the "
-		      "--logfile and --logpriority options.\n");
+			ERROR("To get more details, run the container in foreground mode.");
+		ERROR("Additional information can be obtained by setting the "
+		      "--logfile and --logpriority options.");
 		err = c->error_num;
 		lxc_container_put(c);
 		exit(err);
