@@ -137,7 +137,7 @@ static int do_start(void *arg)
 			exit(EXIT_FAILURE);
 		}
 
-	// Setuid is useful even without a new user id space
+	/* Setuid is useful even without a new user id space. */
 	if (start_arg->setuid && setuid(uid)) {
 		ERROR("failed to set uid %d: %s", uid, strerror(errno));
 		exit(EXIT_FAILURE);
@@ -228,9 +228,6 @@ int main(int argc, char *argv[])
 	 *	dest: del + 1 == OUNT|PID
 	 *	src:  del + 3 == NT|PID
 	 */
-	if (!namespaces)
-		usage(argv[0]);
-
 	while ((del = strstr(namespaces, "MOUNT")))
 		memmove(del + 1, del + 3, strlen(del) - 2);
 
