@@ -31,7 +31,6 @@
 #include <grp.h>
 #include <inttypes.h>
 #include <libgen.h>
-#include <pthread.h>
 #include <stddef.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -47,13 +46,32 @@
 
 #include "log.h"
 #include "lxclock.h"
-#include "memory_utils.h"
 #include "namespace.h"
-#include "parse.h"
 #include "utils.h"
 
-#ifndef HAVE_STRLCPY
-#include "include/strlcpy.h"
+#ifndef PR_SET_MM
+#define PR_SET_MM 35
+#endif
+
+#ifndef PR_SET_MM_MAP
+#define PR_SET_MM_MAP 14
+
+struct prctl_mm_map {
+        uint64_t   start_code;
+        uint64_t   end_code;
+        uint64_t   start_data;
+        uint64_t   end_data;
+        uint64_t   start_brk;
+        uint64_t   brk;
+        uint64_t   start_stack;
+        uint64_t   arg_start;
+        uint64_t   arg_end;
+        uint64_t   env_start;
+        uint64_t   env_end;
+        uint64_t   *auxv;
+        uint32_t   auxv_size;
+        uint32_t   exe_fd;
+};
 #endif
 
 #ifndef O_PATH
@@ -90,6 +108,9 @@ static int _recursive_rmdir(char *dirname, dev_t pdev,
 		struct stat mystat;
 		int rc;
 
+		if (!direntp)
+			break;
+
 		if (!strcmp(direntp->d_name, ".") ||
 		    !strcmp(direntp->d_name, ".."))
 			continue;
@@ -248,13 +269,8 @@ char *get_rundir()
 {
 	char *rundir;
 	const char *homedir;
-	struct stat sb;
 
-	if (stat(RUNTIME_PATH, &sb) < 0) {
-		return NULL;
-	}
-
-	if (geteuid() == sb.st_uid || getegid() == sb.st_gid) {
+	if (geteuid() == 0) {
 		rundir = strdup(RUNTIME_PATH);
 		return rundir;
 	}
@@ -454,120 +470,137 @@ const char** lxc_va_arg_list_to_argv_const(va_list ap, size_t skip)
 	return (const char**)lxc_va_arg_list_to_argv(ap, skip, 0);
 }
 
-struct lxc_popen_FILE *lxc_popen(const char *command)
+extern struct lxc_popen_FILE *lxc_popen(const char *command)
 {
-	int ret;
+	struct lxc_popen_FILE *fp = NULL;
+	int parent_end = -1, child_end = -1;
 	int pipe_fds[2];
 	pid_t child_pid;
-	struct lxc_popen_FILE *fp = NULL;
 
-	ret = pipe2(pipe_fds, O_CLOEXEC);
-	if (ret < 0)
+	int r = pipe2(pipe_fds, O_CLOEXEC);
+
+	if (r < 0) {
+		ERROR("pipe2 failure");
 		return NULL;
-
-	child_pid = fork();
-	if (child_pid < 0)
-		goto on_error;
-
-	if (!child_pid) {
-		sigset_t mask;
-
-		close(pipe_fds[0]);
-
-		/* duplicate stdout */
-		if (pipe_fds[1] != STDOUT_FILENO)
-			ret = dup2(pipe_fds[1], STDOUT_FILENO);
-		else
-			ret = fcntl(pipe_fds[1], F_SETFD, 0);
-		if (ret < 0) {
-			close(pipe_fds[1]);
-			_exit(EXIT_FAILURE);
-		}
-
-		/* duplicate stderr */
-		if (pipe_fds[1] != STDERR_FILENO)
-			ret = dup2(pipe_fds[1], STDERR_FILENO);
-		else
-			ret = fcntl(pipe_fds[1], F_SETFD, 0);
-		close(pipe_fds[1]);
-		if (ret < 0)
-			_exit(EXIT_FAILURE);
-
-		/* unblock all signals */
-		ret = sigfillset(&mask);
-		if (ret < 0)
-			_exit(EXIT_FAILURE);
-
-		ret = pthread_sigmask(SIG_UNBLOCK, &mask, NULL);
-		if (ret < 0)
-			_exit(EXIT_FAILURE);
-
-		/* check if /bin/sh exist, otherwise try Android location /system/bin/sh */
-		if (file_exists("/bin/sh"))
-			execl("/bin/sh", "sh", "-c", command, (char *)NULL);
-		else
-			execl("/system/bin/sh", "sh", "-c", command, (char *)NULL);
-
-		_exit(127);
 	}
 
-	close(pipe_fds[1]);
-	pipe_fds[1] = -1;
+	parent_end = pipe_fds[0];
+	child_end = pipe_fds[1];
 
-	fp = malloc(sizeof(*fp));
-	if (!fp)
-		goto on_error;
-	memset(fp, 0, sizeof(*fp));
+	child_pid = fork();
+
+	if (child_pid == 0) {
+		/* child */
+		int child_std_end = STDOUT_FILENO;
+
+		close(parent_end);
+		parent_end = -1;
+
+		if (child_end != child_std_end) {
+			/* dup2() doesn't dup close-on-exec flag */
+			dup2(child_end, child_std_end);
+
+			/* it's safe not to close child_end here
+			 * as it's marked close-on-exec anyway
+			 */
+		} else {
+			/*
+			 * The descriptor is already the one we will use.
+			 * But it must not be marked close-on-exec.
+			 * Undo the effects.
+			 */
+			if (fcntl(child_end, F_SETFD, 0) != 0) {
+				SYSERROR("Failed to remove FD_CLOEXEC from fd.");
+				exit(127);
+			}
+		}
+
+		/*
+		 * Unblock signals.
+		 * This is the main/only reason
+		 * why we do our lousy popen() emulation.
+		 */
+		{
+			sigset_t mask;
+			sigfillset(&mask);
+			sigprocmask(SIG_UNBLOCK, &mask, NULL);
+		}
+
+		execl("/bin/sh", "sh", "-c", command, (char *) NULL);
+		exit(127);
+	}
+
+	/* parent */
+
+	close(child_end);
+	child_end = -1;
+
+	if (child_pid < 0) {
+		ERROR("fork failure");
+		goto error;
+	}
+
+	fp = calloc(1, sizeof(*fp));
+	if (!fp) {
+		ERROR("failed to allocate memory");
+		goto error;
+	}
+
+	fp->f = fdopen(parent_end, "r");
+	if (!fp->f) {
+		ERROR("fdopen failure");
+		goto error;
+	}
 
 	fp->child_pid = child_pid;
-	fp->pipe = pipe_fds[0];
-
-	/* From now on, closing fp->f will also close fp->pipe. So only ever
-	 * call fclose(fp->f).
-	 */
-	fp->f = fdopen(pipe_fds[0], "r");
-	if (!fp->f)
-		goto on_error;
 
 	return fp;
 
-on_error:
-	/* We can only close pipe_fds[0] if fdopen() didn't succeed or wasn't
-	 * called yet. Otherwise the fd belongs to the file opened by fdopen()
-	 * since it isn't dup()ed.
-	 */
-	if (fp && !fp->f && pipe_fds[0] >= 0)
-		close(pipe_fds[0]);
+error:
 
-	if (pipe_fds[1] >= 0)
-		close(pipe_fds[1]);
+	if (fp) {
+		if (fp->f) {
+			fclose(fp->f);
+			parent_end = -1; /* so we do not close it second time */
+		}
 
-	if (fp && fp->f)
-		fclose(fp->f);
-
-	if (fp)
 		free(fp);
+	}
+
+	if (parent_end != -1)
+		close(parent_end);
 
 	return NULL;
 }
 
-int lxc_pclose(struct lxc_popen_FILE *fp)
+extern int lxc_pclose(struct lxc_popen_FILE *fp)
 {
-	pid_t wait_pid;
+	FILE *f = NULL;
+	pid_t child_pid = 0;
 	int wstatus = 0;
+	pid_t wait_pid;
 
-	if (!fp)
+	if (fp) {
+		f = fp->f;
+		child_pid = fp->child_pid;
+		/* free memory (we still need to close file stream) */
+		free(fp);
+		fp = NULL;
+	}
+
+	if (!f || fclose(f)) {
+		ERROR("fclose failure");
 		return -1;
+	}
 
 	do {
-		wait_pid = waitpid(fp->child_pid, &wstatus, 0);
-	} while (wait_pid < 0 && errno == EINTR);
+		wait_pid = waitpid(child_pid, &wstatus, 0);
+	} while (wait_pid == -1 && errno == EINTR);
 
-	fclose(fp->f);
-	free(fp);
-
-	if (wait_pid < 0)
+	if (wait_pid == -1) {
+		ERROR("waitpid failure");
 		return -1;
+	}
 
 	return wstatus;
 }
@@ -651,8 +684,7 @@ char *lxc_string_join(const char *sep, const char **parts, bool use_as_prefix)
 		return NULL;
 
 	if (use_as_prefix)
-		(void)strlcpy(result, sep, result_len + 1);
-
+		strcpy(result, sep);
 	for (p = (char **)parts; *p; p++) {
 		if (p > (char **)parts)
 			strcat(result, sep);
@@ -741,12 +773,10 @@ char *lxc_deslashify(const char *path)
 
 char *lxc_append_paths(const char *first, const char *second)
 {
-	int ret;
-	size_t len;
-	char *result = NULL;
+	size_t len = strlen(first) + strlen(second) + 1;
 	const char *pattern = "%s%s";
+	char *result = NULL;
 
-	len = strlen(first) + strlen(second) + 1;
 	if (second[0] != '/') {
 		len += 1;
 		pattern = "%s/%s";
@@ -756,12 +786,7 @@ char *lxc_append_paths(const char *first, const char *second)
 	if (!result)
 		return NULL;
 
-	ret = snprintf(result, len, pattern, first, second);
-	if (ret < 0 || (size_t)ret >= len) {
-		free(result);
-		return NULL;
-	}
-
+	snprintf(result, len, pattern, first, second);
 	return result;
 }
 
@@ -769,15 +794,12 @@ bool lxc_string_in_list(const char *needle, const char *haystack, char _sep)
 {
 	char *token, *str, *saveptr = NULL;
 	char sep[2] = { _sep, '\0' };
-	size_t len;
 
 	if (!haystack || !needle)
 		return 0;
 
-	len = strlen(haystack);
-	str = alloca(len + 1);
-	(void)strlcpy(str, haystack, len + 1);
-
+	str = alloca(strlen(haystack)+1);
+	strcpy(str, haystack);
 	for (; (token = strtok_r(str, sep, &saveptr)); str = NULL) {
 		if (strcmp(needle, token) == 0)
 			return 1;
@@ -794,15 +816,12 @@ char **lxc_string_split(const char *string, char _sep)
 	size_t result_capacity = 0;
 	size_t result_count = 0;
 	int r, saved_errno;
-	size_t len;
 
 	if (!string)
 		return calloc(1, sizeof(char *));
 
-	len = strlen(string);
-	str = alloca(len + 1);
-	(void)strlcpy(str, string, len + 1);
-
+	str = alloca(strlen(string) + 1);
+	strcpy(str, string);
 	for (; (token = strtok_r(str, sep, &saveptr)); str = NULL) {
 		r = lxc_grow_array((void ***)&result, &result_capacity, result_count + 1, 16);
 		if (r < 0)
@@ -838,15 +857,12 @@ char **lxc_string_split_and_trim(const char *string, char _sep)
 	size_t result_count = 0;
 	int r, saved_errno;
 	size_t i = 0;
-	size_t len;
 
 	if (!string)
 		return calloc(1, sizeof(char *));
 
-	len = strlen(string);
-	str = alloca(len + 1);
-	(void)strlcpy(str, string, len + 1);
-
+	str = alloca(strlen(string)+1);
+	strcpy(str, string);
 	for (; (token = strtok_r(str, sep, &saveptr)); str = NULL) {
 		while (token[0] == ' ' || token[0] == '\t')
 			token++;
@@ -1053,7 +1069,7 @@ bool dir_exists(const char *path)
 
 	ret = stat(path, &sb);
 	if (ret < 0)
-		// could be something other than eexist, just say no
+		/* Could be something other than eexist, just say "no". */
 		return false;
 	return S_ISDIR(sb.st_mode);
 }
@@ -1109,7 +1125,7 @@ int detect_shared_rootfs(void)
 			continue;
 		*p2 = '\0';
 		if (strcmp(p + 1, "/") == 0) {
-			// this is '/'.  is it shared?
+			/* This is '/'. Is it shared? */
 			p = strchr(p2 + 1, ' ');
 			if (p && strstr(p, "shared:")) {
 				fclose(f);
@@ -1175,7 +1191,7 @@ bool detect_ramfs_rootfs(void)
 			continue;
 		*p2 = '\0';
 		if (strcmp(p + 1, "/") == 0) {
-			// this is '/'.  is it the ramfs?
+			/* This is '/'. Is it the ramfs? */
 			p = strchr(p2 + 1, '-');
 			if (p && strncmp(p, "- rootfs rootfs ", 16) == 0) {
 				free(line);
@@ -1386,6 +1402,110 @@ char *get_template_path(const char *t)
 	return tpath;
 }
 
+/*
+ * Sets the process title to the specified title. Note that this may fail if
+ * the kernel doesn't support PR_SET_MM_MAP (kernels <3.18).
+ */
+int setproctitle(char *title)
+{
+	static char *proctitle = NULL;
+	char buf[2048], *tmp;
+	FILE *f;
+	int i, len, ret = 0;
+
+	/* We don't really need to know all of this stuff, but unfortunately
+	 * PR_SET_MM_MAP requires us to set it all at once, so we have to
+	 * figure it out anyway.
+	 */
+	unsigned long start_data, end_data, start_brk, start_code, end_code,
+			start_stack, arg_start, arg_end, env_start, env_end,
+			brk_val;
+	struct prctl_mm_map prctl_map;
+
+	f = fopen_cloexec("/proc/self/stat", "r");
+	if (!f) {
+		return -1;
+	}
+
+	tmp = fgets(buf, sizeof(buf), f);
+	fclose(f);
+	if (!tmp) {
+		return -1;
+	}
+
+	/* Skip the first 25 fields, column 26-28 are start_code, end_code,
+	 * and start_stack */
+	tmp = strchr(buf, ' ');
+	for (i = 0; i < 24; i++) {
+		if (!tmp)
+			return -1;
+		tmp = strchr(tmp+1, ' ');
+	}
+	if (!tmp)
+		return -1;
+
+	i = sscanf(tmp, "%lu %lu %lu", &start_code, &end_code, &start_stack);
+	if (i != 3)
+		return -1;
+
+	/* Skip the next 19 fields, column 45-51 are start_data to arg_end */
+	for (i = 0; i < 19; i++) {
+		if (!tmp)
+			return -1;
+		tmp = strchr(tmp+1, ' ');
+	}
+
+	if (!tmp)
+		return -1;
+
+	i = sscanf(tmp, "%lu %lu %lu %*u %*u %lu %lu",
+		&start_data,
+		&end_data,
+		&start_brk,
+		&env_start,
+		&env_end);
+	if (i != 5)
+		return -1;
+
+	/* Include the null byte here, because in the calculations below we
+	 * want to have room for it. */
+	len = strlen(title) + 1;
+
+	proctitle = realloc(proctitle, len);
+	if (!proctitle)
+		return -1;
+
+	arg_start = (unsigned long) proctitle;
+	arg_end = arg_start + len;
+
+	brk_val = syscall(__NR_brk, 0);
+
+	prctl_map = (struct prctl_mm_map) {
+		.start_code = start_code,
+		.end_code = end_code,
+		.start_stack = start_stack,
+		.start_data = start_data,
+		.end_data = end_data,
+		.start_brk = start_brk,
+		.brk = brk_val,
+		.arg_start = arg_start,
+		.arg_end = arg_end,
+		.env_start = env_start,
+		.env_end = env_end,
+		.auxv = NULL,
+		.auxv_size = 0,
+		.exe_fd = -1,
+	};
+
+	ret = prctl(PR_SET_MM, PR_SET_MM_MAP, (long) &prctl_map, sizeof(prctl_map), 0);
+	if (ret == 0)
+		strcpy((char*)arg_start, title);
+	else
+		INFO("setting cmdline failed - %s", strerror(errno));
+
+	return ret;
+}
+
 /*
  * @path:    a pathname where / replaced with '\0'.
  * @offsetp: pointer to int showing which path segment was last seen.
@@ -1452,20 +1572,21 @@ static int check_symlink(int fd)
 static int open_if_safe(int dirfd, const char *nextpath)
 {
 	int newfd = openat(dirfd, nextpath, O_RDONLY | O_NOFOLLOW);
-	if (newfd >= 0) // was not a symlink, all good
+	if (newfd >= 0) /* Was not a symlink, all good. */
 		return newfd;
 
 	if (errno == ELOOP)
 		return newfd;
 
 	if (errno == EPERM || errno == EACCES) {
-		/* we're not root (cause we got EPERM) so
-		   try opening with O_PATH */
+		/* We're not root (cause we got EPERM) so try opening with
+		 * O_PATH.
+		 */
 		newfd = openat(dirfd, nextpath, O_PATH | O_NOFOLLOW);
 		if (newfd >= 0) {
-			/* O_PATH will return an fd for symlinks.  We know
-			 * nextpath wasn't a symlink at last openat, so if fd
-			 * is now a link, then something * fishy is going on
+			/* O_PATH will return an fd for symlinks. We know
+			 * nextpath wasn't a symlink at last openat, so if fd is
+			 * now a link, then something * fishy is going on.
 			 */
 			int ret = check_symlink(newfd);
 			if (ret < 0) {
@@ -1565,8 +1686,10 @@ out:
 int safe_mount(const char *src, const char *dest, const char *fstype,
 		unsigned long flags, const void *data, const char *rootfs)
 {
-	int srcfd = -1, destfd, ret, saved_errno;
-	char srcbuf[50], destbuf[50]; // only needs enough for /proc/self/fd/<fd>
+	int destfd, ret, saved_errno;
+	/* Only needs enough for /proc/self/fd/<fd>. */
+	char srcbuf[50], destbuf[50];
+	int srcfd = -1;
 	const char *mntsrc = src;
 
 	if (!rootfs)
@@ -1613,7 +1736,7 @@ int safe_mount(const char *src, const char *dest, const char *fstype,
 	close(destfd);
 	if (ret < 0) {
 		errno = saved_errno;
-		SYSERROR("Failed to mount %s onto %s", src ? src : "(null)", dest);
+		SYSERROR("Failed to mount %s onto %s", src, dest);
 		return ret;
 	}
 
@@ -1663,7 +1786,7 @@ int lxc_mount_proc_if_needed(const char *rootfs)
 		return -1;
 	}
 
-	mypid = lxc_raw_getpid();
+	mypid = getpid();
 	INFO("I am %d, /proc/self points to \"%s\"", mypid, link);
 
 	if (lxc_safe_int(link, &link_to_pid) < 0)
@@ -1786,16 +1909,17 @@ int lxc_strmunmap(void *addr, size_t length)
 
 /* Check whether a signal is blocked by a process. */
 /* /proc/pid-to-str/status\0 = (5 + 21 + 7 + 1) */
-#define __PROC_STATUS_LEN (6 + (LXC_NUMSTRLEN64) + 7 + 1)
-bool task_blocks_signal(pid_t pid, int signal)
+#define __PROC_STATUS_LEN (5 + (LXC_NUMSTRLEN64) + 7 + 1)
+bool task_blocking_signal(pid_t pid, int signal)
 {
-	int ret;
-	char status[__PROC_STATUS_LEN];
-	FILE *f;
-	uint64_t sigblk = 0, one = 1;
-	size_t n = 0;
 	bool bret = false;
 	char *line = NULL;
+	long unsigned int sigblk = 0;
+	size_t n = 0;
+	int ret;
+	FILE *f;
+
+	char status[__PROC_STATUS_LEN];
 
 	ret = snprintf(status, __PROC_STATUS_LEN, "/proc/%d/status", pid);
 	if (ret < 0 || ret >= __PROC_STATUS_LEN)
@@ -1806,20 +1930,14 @@ bool task_blocks_signal(pid_t pid, int signal)
 		return bret;
 
 	while (getline(&line, &n, f) != -1) {
-		char *numstr;
-
-		if (strncmp(line, "SigBlk:", 7))
+		if (strncmp(line, "SigBlk:\t", 8))
 			continue;
 
-		numstr = lxc_trim_whitespace_in_place(line + 7);
-		ret = lxc_safe_uint64(numstr, &sigblk, 16);
-		if (ret < 0)
+		if (sscanf(line + 8, "%lx", &sigblk) != 1)
 			goto out;
-
-		break;
 	}
 
-	if (sigblk & (one << (signal - 1)))
+	if (sigblk & (1LU << (signal - 1)))
 		bret = true;
 
 out:
@@ -1880,9 +1998,8 @@ int lxc_preserve_ns(const int pid, const char *ns)
 	ret = snprintf(path, __NS_PATH_LEN, "/proc/%d/ns%s%s", pid,
 		       !ns || strcmp(ns, "") == 0 ? "" : "/",
 		       !ns || strcmp(ns, "") == 0 ? "" : ns);
-	errno = EFBIG;
 	if (ret < 0 || (size_t)ret >= __NS_PATH_LEN)
-		return -EFBIG;
+		return -1;
 
 	return open(path, O_RDONLY | O_CLOEXEC);
 }
@@ -1936,29 +2053,6 @@ int lxc_safe_ulong(const char *numstr, unsigned long *converted)
 	return 0;
 }
 
-int lxc_safe_uint64(const char *numstr, uint64_t *converted, int base)
-{
-	char *err = NULL;
-	uint64_t u;
-
-	while (isspace(*numstr))
-		numstr++;
-
-	if (*numstr == '-')
-		return -EINVAL;
-
-	errno = 0;
-	u = strtoull(numstr, &err, base);
-	if (errno == ERANGE && u == ULLONG_MAX)
-		return -ERANGE;
-
-	if (err == numstr || *err != '\0')
-		return -EINVAL;
-
-	*converted = u;
-	return 0;
-}
-
 int lxc_safe_int(const char *numstr, int *converted)
 {
 	char *err = NULL;
@@ -2002,26 +2096,6 @@ int lxc_safe_long(const char *numstr, long int *converted)
 	return 0;
 }
 
-int lxc_safe_long_long(const char *numstr, long long int *converted)
-{
-	char *err = NULL;
-	signed long long int sli;
-
-	errno = 0;
-	sli = strtoll(numstr, &err, 0);
-	if (errno == ERANGE && (sli == LLONG_MAX || sli == LLONG_MIN))
-		return -ERANGE;
-
-	if (errno != 0 && sli == 0)
-		return -EINVAL;
-
-	if (err == numstr || *err != '\0')
-		return -EINVAL;
-
-	*converted = sli;
-	return 0;
-}
-
 int lxc_switch_uid_gid(uid_t uid, gid_t gid)
 {
 	if (setgid(gid) < 0) {
@@ -2063,6 +2137,9 @@ static int lxc_get_unused_loop_dev_legacy(char *loop_name)
 		return -1;
 
 	while ((dp = readdir(dir))) {
+		if (!dp)
+			break;
+
 		if (strncmp(dp->d_name, "loop", 4) != 0)
 			continue;
 
@@ -2208,7 +2285,7 @@ int run_command(char *buf, size_t buf_size, int (*child_fn)(void *), void *args)
 	int ret, fret, pipefd[2];
 	ssize_t bytes;
 
-	/* Make sure our callers do not receive uninitialized memory. */
+	/* Make sure our callers do not receive unitialized memory. */
 	if (buf_size > 0 && buf)
 		buf[0] = '\0';
 
@@ -2217,7 +2294,7 @@ int run_command(char *buf, size_t buf_size, int (*child_fn)(void *), void *args)
 		return -1;
 	}
 
-	child = lxc_raw_clone(0);
+	child = fork();
 	if (child < 0) {
 		close(pipefd[0]);
 		close(pipefd[1]);
@@ -2241,23 +2318,21 @@ int run_command(char *buf, size_t buf_size, int (*child_fn)(void *), void *args)
 
 		if (ret < 0) {
 			SYSERROR("failed to duplicate std{err,out} file descriptor");
-			_exit(EXIT_FAILURE);
+			exit(EXIT_FAILURE);
 		}
 
 		/* Does not return. */
 		child_fn(args);
 		ERROR("failed to exec command");
-		_exit(EXIT_FAILURE);
+		exit(EXIT_FAILURE);
 	}
 
 	/* close the write-end of the pipe */
 	close(pipefd[1]);
 
-	if (buf && buf_size > 0) {
-		bytes = read(pipefd[0], buf, buf_size - 1);
-		if (bytes > 0)
-			buf[bytes - 1] = '\0';
-	}
+	bytes = read(pipefd[0], buf, (buf_size > 0) ? (buf_size - 1) : 0);
+	if (bytes > 0)
+		buf[bytes - 1] = '\0';
 
 	fret = wait_for_pid(child);
 	/* close the read-end of the pipe */
@@ -2289,33 +2364,6 @@ char *must_make_path(const char *first, ...)
 	return dest;
 }
 
-char *must_append_path(char *first, ...)
-{
-	char *cur;
-	size_t full_len;
-	va_list args;
-	char *dest = first;
-
-	full_len = strlen(first);
-	va_start(args, first);
-	while ((cur = va_arg(args, char *)) != NULL) {
-		full_len += strlen(cur);
-
-		if (cur[0] != '/')
-			full_len++;
-
-		dest = must_realloc(dest, full_len + 1);
-
-		if (cur[0] != '/')
-			strcat(dest, "/");
-
-		strcat(dest, cur);
-	}
-	va_end(args);
-
-	return dest;
-}
-
 char *must_copy_string(const char *entry)
 {
 	char *ret;
@@ -2362,18 +2410,6 @@ bool has_fs_type(const char *path, fs_type_magic magic_val)
 	return has_type;
 }
 
-bool fhas_fs_type(int fd, fs_type_magic magic_val)
-{
-	int ret;
-	struct statfs sb;
-
-	ret = fstatfs(fd, &sb);
-	if (ret < 0)
-		return false;
-
-	return is_fs_type(&sb, magic_val);
-}
-
 bool lxc_nic_exists(char *nic)
 {
 #define __LXC_SYS_CLASS_NET_LEN 15 + IFNAMSIZ + 1
@@ -2394,108 +2430,3 @@ bool lxc_nic_exists(char *nic)
 
 	return true;
 }
-
-int lxc_make_tmpfile(char *template, bool rm)
-{
-	__do_close_prot_errno int fd = -EBADF;
-	int ret;
-	mode_t msk;
-
-	msk = umask(0022);
-	fd = mkstemp(template);
-	umask(msk);
-	if (fd < 0)
-		return -1;
-
-	if (lxc_set_cloexec(fd))
-		return -1;
-
-	if (!rm)
-		return move_fd(fd);
-
-	ret = unlink(template);
-	if (ret < 0)
-		return -1;
-
-	return move_fd(fd);
-}
-
-int parse_byte_size_string(const char *s, int64_t *converted)
-{
-	int ret, suffix_len;
-	long long int conv;
-	int64_t mltpl, overflow;
-	char *end;
-	char dup[LXC_NUMSTRLEN64 + 2];
-	char suffix[3] = {0};
-
-	if (!s || !strcmp(s, ""))
-		return -EINVAL;
-
-	end = stpncpy(dup, s, sizeof(dup) - 1);
-	if (*end != '\0')
-		return -EINVAL;
-
-	if (isdigit(*(end - 1)))
-		suffix_len = 0;
-	else if (isalpha(*(end - 1)))
-		suffix_len = 1;
-	else
-		return -EINVAL;
-
-	if (suffix_len > 0 && (end - 2) == dup && !isdigit(*(end - 2)))
-		return -EINVAL;
-
-	if (suffix_len > 0 && isalpha(*(end - 2)))
-		suffix_len++;
-
-	if (suffix_len > 0) {
-		memcpy(suffix, end - suffix_len, suffix_len);
-		*(suffix + suffix_len) = '\0';
-		*(end - suffix_len) = '\0';
-	}
-	dup[lxc_char_right_gc(dup, strlen(dup))] = '\0';
-
-	ret = lxc_safe_long_long(dup, &conv);
-	if (ret < 0)
-		return -ret;
-
-	if (suffix_len != 2) {
-		*converted = conv;
-		return 0;
-	}
-
-	if (!strcmp(suffix, "kB"))
-		mltpl = 1024;
-	else if (!strcmp(suffix, "MB"))
-		mltpl = 1024 * 1024;
-	else if (!strcmp(suffix, "GB"))
-		mltpl = 1024 * 1024 * 1024;
-	else
-		return -EINVAL;
-
-	overflow = conv * mltpl;
-	if (conv != 0 && (overflow / conv) != mltpl)
-		return -ERANGE;
-
-	*converted = overflow;
-	return 0;
-}
-
-uint64_t lxc_find_next_power2(uint64_t n)
-{
-	/* 0 is not valid input. We return 0 to the caller since 0 is not a
-	 * valid power of two.
-	 */
-	if (n == 0)
-		return 0;
-
-	if (!(n & (n - 1)))
-		return n;
-
-	while (n & (n - 1))
-		n = n & (n - 1);
-
-	n = n << 1;
-	return n;
-}
