@@ -85,8 +85,8 @@ static void do_function(void *arguments)
     }
 
     if (debug) {
-        c->set_config_item(c, "lxc.loglevel", "DEBUG");
-        c->set_config_item(c, "lxc.logfile", name);
+        c->set_config_item(c, "lxc.log.level", "DEBUG");
+        c->set_config_item(c, "lxc.log", name);
     }
 
     if (strcmp(args->mode, "create") == 0) {
@@ -177,9 +177,6 @@ int main(int argc, char *argv[]) {
         case 'm': {
             char *mode_tok, *tok, *saveptr = NULL;
 
-	    if (!optarg)
-                continue;
-
             modes = NULL;
             for (i = 0, mode_tok = optarg;
                  (tok = strtok_r(mode_tok, ",", &saveptr));
@@ -191,8 +188,7 @@ int main(int argc, char *argv[]) {
                 }
                 modes[i] = tok;
             }
-	    if (modes)
-		    modes[i] = NULL;
+            modes[i] = NULL;
             break;
         }
         default: /* '?' */
