@@ -40,8 +40,10 @@ static int destroy_container(void)
 		return -1;
 	}
 	if (pid == 0) {
-		execlp("lxc-destroy", "lxc-destroy", "-f", "-n", MYNAME, NULL);
-		exit(EXIT_FAILURE);
+		ret = execlp("lxc-destroy", "lxc-destroy", "-f", "-n", MYNAME, NULL);
+		// Should not return
+		perror("execl");
+		exit(1);
 	}
 again:
 	ret = waitpid(pid, &status, 0);
@@ -70,8 +72,10 @@ static int create_container(void)
 		return -1;
 	}
 	if (pid == 0) {
-		execlp("lxc-create", "lxc-create", "-t", "busybox", "-n", MYNAME, NULL);
-		exit(EXIT_FAILURE);
+		ret = execlp("lxc-create", "lxc-create", "-t", "busybox", "-n", MYNAME, NULL);
+		// Should not return
+		perror("execl");
+		exit(1);
 	}
 again:
 	ret = waitpid(pid, &status, 0);
@@ -150,8 +154,8 @@ int main(int argc, char *argv[])
 		goto out;
 	}
 
-	if (!c->set_config_item(c, "lxc.utsname", "bobo")) {
-		fprintf(stderr, "%d: failed setting lxc.utsname\n", __LINE__);
+	if (!c->set_config_item(c, "lxc.uts.name", "bobo")) {
+		fprintf(stderr, "%d: failed setting lxc.uts.name\n", __LINE__);
 		goto out;
 	}
 
